/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.Assembler;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public abstract class AbstractAssembler implements Assembler {

	public AbstractAssembler(ABI abi) {
		this.abi = abi;
	}

	public AbstractAssembler(AbstractAssembler prototype) {
		this.abi = prototype.abi;
		this.sourceDirectory = prototype.sourceDirectory;
		this.objectDirectory = prototype.objectDirectory;
		this.macros = new HashMap<String, String>(prototype.macros);
		this.includeDirectories = new HashSet<AbsoluteDirectoryPath>(prototype.includeDirectories);
		this.debugInformation = prototype.debugInformation;
		this.verboseBuild = prototype.verboseBuild;
	}

	protected final String getExtension() {
		if (getABI().getOperatingSystem().equals(OperatingSystem.Windows)) {
			return "obj";
		} else {
			return "o";
		}
	}

	@Override
	public AbsoluteFilePath getObjectPath(AbsoluteFilePath sourcePath) {
		if (sourceDirectory == null) {
			throw new IllegalStateException("Source directory is not initialized");
		}
		if (objectDirectory == null) {
			throw new IllegalStateException("Object directory is not initialized");
		}
		final RelativeFilePath relativeSourcePath = sourcePath.getRelativePath(sourceDirectory);
		if (relativeSourcePath == null) {
			throw new IllegalArgumentException("Source path is not within source directory");
		}
		return new AbsoluteFilePath(objectDirectory,  relativeSourcePath.replaceExtension(getExtension()));
	}

	@Override
	public void setSourceDirectory(AbsoluteDirectoryPath sourceDirectory) {
		this.sourceDirectory = sourceDirectory;
	}

	@Override
	public AbsoluteDirectoryPath getSourceDirectory() {
		return this.sourceDirectory;
	}

	@Override
	public void setObjectDirectory(AbsoluteDirectoryPath objectDirectory) {
		this.objectDirectory = objectDirectory;
	}

	@Override
	public AbsoluteDirectoryPath getObjectDirectory() {
		return this.objectDirectory;
	}

	@Override
	public void setVerboseBuild(boolean verboseBuild) {
		this.verboseBuild = verboseBuild;
	}

	@Override
	public boolean isVerboseBuild() {
		return this.verboseBuild;
	}

	@Override
	public void setDebugInformation(boolean debugInformation) {
		this.debugInformation = debugInformation;
	}

	@Override
	public boolean getDebugInformation() {
		return this.debugInformation;
	}

	@Override
	public boolean addIncludeDirectory(AbsoluteDirectoryPath includePath) {
		return includeDirectories.add(includePath);
	}

	@Override
	public boolean removeIncludeDirectory(AbsoluteDirectoryPath includePath) {
		return this.includeDirectories.remove(includePath);
	}

	@Override
	public void clearIncludeDirectories() {
		this.includeDirectories.clear();
	}

	@Override
	public boolean addMacro(String name) {
		return this.addMacro(name, "1");
	}

	@Override
	public boolean addMacro(String name, String value) {
		if (macros.containsKey(name)) {
			return false;
		} else {
			macros.put(name, value);
			return true;
		}
	}

	@Override
	public boolean removeMacro(String name) {
		return (macros.remove(name) != null);
	}

	@Override
	public void clearMacros() {
		this.macros.clear();
	}

	@Override
	public void clearAll() {
		this.clearIncludeDirectories();
		this.clearMacros();
	}

	@Override
	public ABI getABI() {
		return this.abi;
	}

	private final ABI abi;
	protected AbsoluteDirectoryPath sourceDirectory = null;
	protected AbsoluteDirectoryPath objectDirectory = null;
	protected Map<String, String> macros = new HashMap<String, String>();
	protected Set<AbsoluteDirectoryPath> includeDirectories = new HashSet<AbsoluteDirectoryPath>();
	protected boolean debugInformation = true;
	protected boolean verboseBuild = false;

}
