/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.Versioned;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AndroidNDK implements Versioned {
	
	private static final Pattern versionRegex = Pattern.compile("(r(\\d+)([a-z]?))(?:\\s.+)?");

	public static VersionList<AndroidNDK> enumerate(Machine machine) {
		final VersionList<AndroidNDK> products = new VersionList<AndroidNDK>();

		if (machine.isLocal()) {
			try {
				final AbsoluteDirectoryPath rootPath = new AbsoluteDirectoryPath(machine, System.getenv("ANDROID_NDK_ROOT"));
				if (new AbsoluteFilePath(rootPath, new RelativeFilePath("ndk-build")).exists()) {
					final String releaseName = new AbsoluteFilePath(rootPath, new RelativeFilePath("RELEASE.TXT")).readFirstLine();
					final Matcher versionMatcher = versionRegex.matcher(releaseName);
					if (versionMatcher.matches()) {
						final int majorVersion = Integer.parseInt(versionMatcher.group(2));
						final String versionLetter = versionMatcher.group(3);
						final int minorVersion = (versionLetter.isEmpty()) ? 0 : (versionLetter.charAt(0) - '0') + 1;
						final String versionName = versionMatcher.group(1);
						final Version version = new Version(majorVersion, minorVersion);
						products.add(new AndroidNDK(rootPath, version, versionName));
					}
				}
			} catch (Exception e) {
			}
		}
		return products;
	}

	public VersionList<AndroidPlatform> enumeratePlatforms(ABI abi) {
		return AndroidPlatform.enumerate(getDirectory(), abi);
	}

	public List<AndroidToolchain> enumerateToolchains(ABI abi) {
		final AndroidPlatform defaultPlatform = getDefaultPlatform(abi);
		return AndroidToolchain.enumerate(getDirectory(), abi, defaultPlatform);
	}

	public VersionList<AndroidToolchain> enumerateToolchains(ABI abi, AndroidToolchain.Type type) {
		final AndroidPlatform defaultPlatform = getDefaultPlatform(abi);
		return AndroidToolchain.enumerate(getDirectory(), abi, type, defaultPlatform);
	}

	public AndroidNDK(AbsoluteDirectoryPath rootDirectory, Version version, String versionName) {
		this.rootDirectory = rootDirectory;
		this.version = version;
		this.versionName = versionName;
		this.defaultPlatforms = new HashMap<ABI, AndroidPlatform>(4);
		defaultPlatforms.put(ABI.ARM_Linux_SoftEABI_Android, AndroidPlatform.enumerate(rootDirectory, ABI.ARM_Linux_SoftEABI_Android).getOldest());
		defaultPlatforms.put(ABI.ARM_Linux_SoftEABI_AndroidV7A, AndroidPlatform.enumerate(rootDirectory, ABI.ARM_Linux_SoftEABI_AndroidV7A).getOldest());
		defaultPlatforms.put(ABI.X86_Linux_Pic_Android, AndroidPlatform.enumerate(rootDirectory, ABI.X86_Linux_Pic_Android).getOldest(new Version(9)));
		defaultPlatforms.put(ABI.MIPS_Linux_O32_Android, AndroidPlatform.enumerate(rootDirectory, ABI.MIPS_Linux_O32_Android).getOldest(new Version(9)));
	}

	@Override
	public Version getVersion() {
		return this.version;
	}

	public AbsoluteDirectoryPath getDirectory() {
		return this.rootDirectory;
	}

	public AndroidPlatform getDefaultPlatform(ABI abi) {
		return defaultPlatforms.get(abi);
	}

	public void setDefaultPlatform(AndroidPlatform defaultPlatform) {
		final ABI abi = defaultPlatform.getABI();
		defaultPlatforms.put(abi, defaultPlatform);
	}

	@Override
	public String toString() {
		return "Android NDK " + versionName;
	}

	private final String versionName;
	private final Version version;
	private final AbsoluteDirectoryPath rootDirectory;
	private final Map<ABI, AndroidPlatform> defaultPlatforms;
}
