/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.Versioned;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AndroidPlatform implements Versioned {

	final private static Pattern versionDirectoryPattern = Pattern.compile("android\\-(\\d+)");
	final private static Pattern abiDirectoryPattern = Pattern.compile("arch\\-(arm|mips|x86)");
	final private static Pattern armAbiDirectoryPattern = Pattern.compile(Pattern.quote("arch-arm"));
	final private static Pattern x86AbiDirectoryPattern = Pattern.compile(Pattern.quote("arch-x86"));
	final private static Pattern mipsAbiDirectoryPattern = Pattern.compile(Pattern.quote("arch-mips"));

	public static List<AndroidPlatform> enumerate(AbsoluteDirectoryPath ndkDirectory) {
		final List<AndroidPlatform> platforms = new LinkedList<AndroidPlatform>();
		final AbsoluteDirectoryPath platformsDirectory = new AbsoluteDirectoryPath(ndkDirectory, new RelativeDirectoryPath("platforms"));
		final List<AbsoluteDirectoryPath> platformVersionDirectories = platformsDirectory.getDirectories(versionDirectoryPattern);
		for (final AbsoluteDirectoryPath platformVersionDirectory : platformVersionDirectories) {
			final Matcher platformVersionMatcher = versionDirectoryPattern.matcher(platformVersionDirectory.getLastFileName());
			platformVersionMatcher.matches();
			final Version version = new Version(platformVersionMatcher.group(1));
			final List<AbsoluteDirectoryPath> platformAbiDirectories = platformVersionDirectory.getDirectories(abiDirectoryPattern);
			for (final AbsoluteDirectoryPath platformAbiDirectory : platformAbiDirectories) {
				if (new AbsoluteFilePath(platformAbiDirectory, new RelativeFilePath("usr/include/android/api-level.h")).exists()) {
					if (new AbsoluteFilePath(platformAbiDirectory, new RelativeFilePath("usr/lib/libc.so")).exists()) {
						final Matcher platformAbiMatcher = abiDirectoryPattern.matcher(platformAbiDirectory.getLastFileName());
						platformAbiMatcher.matches();
						final String abiName = platformAbiMatcher.group(1);
						if (abiName.equals("arm")) {
							platforms.add(new AndroidPlatform(platformAbiDirectory, ABI.ARM_Linux_SoftEABI_Android, version));
							platforms.add(new AndroidPlatform(platformAbiDirectory, ABI.ARM_Linux_SoftEABI_AndroidV7A, version));
						} else if (abiName.equals("mips")) {
							platforms.add(new AndroidPlatform(platformAbiDirectory, ABI.MIPS_Linux_O32_Android, version));
						} else if (abiName.equals("x86")) {
							platforms.add(new AndroidPlatform(platformAbiDirectory, ABI.X86_Linux_Pic_Android, version));
						} else {
							throw new Error(String.format("Unknown ABI name %s", abiName));
						}
					}
				}
			}
		}
		return platforms;
	}

	private static Pattern getAbiDirectoryPattern(ABI abi) {
		switch (abi) {
			case ARM_Linux_SoftEABI_Android:
			case ARM_Linux_SoftEABI_AndroidV7A:
				return armAbiDirectoryPattern;
			case X86_Linux_Pic_Android:
				return x86AbiDirectoryPattern;
			case MIPS_Linux_O32_Android:
				return mipsAbiDirectoryPattern;
			default:
				throw new Error(String.format("Unsupported ABI %s", abi.toString()));
		}
	}

	public static VersionList<AndroidPlatform> enumerate(AbsoluteDirectoryPath ndkDirectory, ABI abi) {
		final VersionList<AndroidPlatform> platforms = new VersionList<AndroidPlatform>();
		final AbsoluteDirectoryPath platformsDirectory = new AbsoluteDirectoryPath(ndkDirectory, new RelativeDirectoryPath("platforms"));
		final List<AbsoluteDirectoryPath> platformVersionDirectories = platformsDirectory.getDirectories(versionDirectoryPattern);
		final Pattern abiDirectoryPattern = getAbiDirectoryPattern(abi);
		for (final AbsoluteDirectoryPath platformVersionDirectory : platformVersionDirectories) {
			final Matcher platformVersionMatcher = versionDirectoryPattern.matcher(platformVersionDirectory.getLastFileName());
			platformVersionMatcher.matches();
			final Version version = new Version(platformVersionMatcher.group(1));
			final List<AbsoluteDirectoryPath> platformAbiDirectories = platformVersionDirectory.getDirectories(abiDirectoryPattern);
			for (final AbsoluteDirectoryPath platformAbiDirectory : platformAbiDirectories) {
				if (new AbsoluteFilePath(platformAbiDirectory, new RelativeFilePath("usr/include/android/api-level.h")).exists()) {
					if (new AbsoluteFilePath(platformAbiDirectory, new RelativeFilePath("usr/lib/libc.so")).exists()) {
						platforms.add(new AndroidPlatform(platformAbiDirectory, abi, version));
					}
				}
			}
		}
		return platforms;
	}

	public ABI getABI() {
		return this.abi;
	}

	@Override
	public Version getVersion() {
		return this.version;
	}

	public AbsoluteDirectoryPath getSysRoot() {
		return this.sysRoot;
	}

	public AbsoluteDirectoryPath getIncludeDirectory() {
		return new AbsoluteDirectoryPath(this.sysRoot, new RelativeDirectoryPath("usr/include"));
	}

	public AbsoluteDirectoryPath getLibraryDirectory() {
		return new AbsoluteDirectoryPath(this.sysRoot, new RelativeDirectoryPath("usr/lib"));
	}

	public AndroidPlatform(AbsoluteDirectoryPath sysRoot, ABI abi, Version version) {
		this.sysRoot = sysRoot;
		this.abi = abi;
		this.version = version;
	}

	@Override
	public String toString() {
		return String.format("Android platform version %s for %s", getVersion().toString(), getABI().toString());
	}

	private final AbsoluteDirectoryPath sysRoot;
	private final ABI abi;
	private final Version version;

}