/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.*;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.MatchResult;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AndroidToolchain implements Versioned {

	public static enum Type {
		GNU,
		LLVM
	}

	private static final Pattern gnuToolchainPattern = Pattern.compile("(arm|mipsel|x86)\\-(?:\\w+\\-)*(\\d+(?:\\.\\d+)*)");

	public static List<AndroidToolchain> enumerate(AbsoluteDirectoryPath ndkDirectory, AndroidPlatform platform) {
		final List<AndroidToolchain> products = new LinkedList<AndroidToolchain>();

		try {
			final AbsoluteDirectoryPath toolchainDirectory = new AbsoluteDirectoryPath(ndkDirectory, new RelativeDirectoryPath("toolchains"));
			final List<AbsoluteDirectoryPath> directories = toolchainDirectory.getDirectories(gnuToolchainPattern);
			for (AbsoluteDirectoryPath directory : directories) {
				final MatchResult matcher = gnuToolchainPattern.matcher(directory.getLastFileName()).toMatchResult();
				final String abiName = matcher.group(1);
				final String versionString = matcher.group(2);
				final Version version = new Version(versionString);
				if (abiName.equals("arm")) {
					products.add(new AndroidToolchain(directory, ABI.ARM_Linux_SoftEABI_Android, version, Type.GNU, platform));
					products.add(new AndroidToolchain(directory, ABI.ARM_Linux_SoftEABI_AndroidV7A, version, Type.GNU, platform));
				} else if (abiName.equals("mipsel")) {
					products.add(new AndroidToolchain(directory, ABI.MIPS_Linux_O32_Android, version, Type.GNU, platform));
				} else if (abiName.equals("x86")) {
					products.add(new AndroidToolchain(directory, ABI.X86_Linux_Pic_Android, version, Type.GNU, platform));
				} else {
					throw new Error(String.format("Unknown abi name for gcc toolchain: %s", abiName));
				}
			}
		} catch (Exception e) {
		}
		return products;
	}

	public static List<AndroidToolchain> enumerate(AbsoluteDirectoryPath ndkDirectory, ABI abi, AndroidPlatform platform) {
		final List<AndroidToolchain> products = new LinkedList<AndroidToolchain>();

		try {
			final AbsoluteDirectoryPath toolchainDirectory = new AbsoluteDirectoryPath(ndkDirectory, new RelativeDirectoryPath("toolchains"));
			final List<AbsoluteDirectoryPath> directories = toolchainDirectory.getDirectories(gnuToolchainPattern);
			for (AbsoluteDirectoryPath directory : directories) {
				final MatchResult matcher = gnuToolchainPattern.matcher(directory.getLastFileName()).toMatchResult();
				final String abiName = matcher.group(1);
				final String versionString = matcher.group(2);
				final Version version = new Version(versionString);
				if (abiName.equals("arm")) {
					if (abi.equals(ABI.ARM_Linux_SoftEABI_Android)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					} else if (abi.equals(ABI.ARM_Linux_SoftEABI_AndroidV7A)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					}
				} else if (abiName.equals("mipsel")) {
					if (abi.equals(ABI.MIPS_Linux_O32_Android)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					}
				} else if (abiName.equals("x86")) {
					if (abi.equals(ABI.X86_Linux_Pic_Android)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					}
				} else {
					throw new Error(String.format("Unknown abi name for gcc toolchain: %s", abiName));
				}
			}
		} catch (Exception e) {
		}
		return products;
	}

	public static VersionList<AndroidToolchain> enumerate(AbsoluteDirectoryPath ndkDirectory, ABI abi, Type type, AndroidPlatform platform) {
		final VersionList<AndroidToolchain> products = new VersionList<AndroidToolchain>();

		try {
			final AbsoluteDirectoryPath toolchainDirectory = new AbsoluteDirectoryPath(ndkDirectory, new RelativeDirectoryPath("toolchains"));
			final List<AbsoluteDirectoryPath> directories = toolchainDirectory.getDirectories(gnuToolchainPattern);
			for (AbsoluteDirectoryPath directory : directories) {
				final Matcher matcher = gnuToolchainPattern.matcher(directory.getLastFileName());
				matcher.matches();
				final String abiName = matcher.group(1);
				final String versionString = matcher.group(2);
				final Version version = new Version(versionString);
				if (abiName.equals("arm")) {
					if (abi.equals(ABI.ARM_Linux_SoftEABI_Android)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					} else if (abi.equals(ABI.ARM_Linux_SoftEABI_AndroidV7A)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					}
				} else if (abiName.equals("mipsel")) {
					if (abi.equals(ABI.MIPS_Linux_O32_Android)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					}
				} else if (abiName.equals("x86")) {
					if (abi.equals(ABI.X86_Linux_Pic_Android)) {
						products.add(new AndroidToolchain(directory, abi, version, Type.GNU, platform));
					}
				} else {
					throw new Error(String.format("Unknown abi name for gcc toolchain: %s", abiName));
				}
			}
		} catch (Exception e) {
		}
		return products;
	}

	private final String getOsName(OperatingSystem operatingSystem, boolean is64Bit) {
		switch (operatingSystem) {
			case Windows:
				return is64Bit ? "windows-x86_64" : "windows";
			case Linux:
				return is64Bit ? "linux-x86_64" : "linux-x86";
			case MacOSX:
				return is64Bit ? "darwin-x86_64" : "darwin-x86";
			default:
				throw new Error(String.format("Unknown OS for Android NDK host: %s", operatingSystem.toString()));
		}
	}

	private AbsoluteDirectoryPath getBinariesDirectory(AbsoluteDirectoryPath toolchainDirectory) {
		final boolean is64BitOs = toolchainDirectory.getMachine().getArchitecture().is64bit();
		final OperatingSystem operatingSystem = toolchainDirectory.getMachine().getOperatingSystem();
		final AbsoluteDirectoryPath binaries32Directory = new AbsoluteDirectoryPath(toolchainDirectory, new RelativeDirectoryPath("prebuilt/" + getOsName(operatingSystem, false) + "/bin"));
		if (is64BitOs) {
			final AbsoluteDirectoryPath binaries64Directory = new AbsoluteDirectoryPath(toolchainDirectory, new RelativeDirectoryPath("prebuilt/" + getOsName(operatingSystem, true) + "/bin"));
			if (binaries64Directory.exists()) {
				return binaries64Directory;
			}
		}
		return binaries32Directory;
	}

    private AbsoluteFilePath getCppCompilerPath(AbsoluteDirectoryPath toolchainDirectory, ABI abi, Type type) {
	    final String executableSuffix = toolchainDirectory.getMachine().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : new String();
	    final AbsoluteDirectoryPath binariesDirectory = getBinariesDirectory(toolchainDirectory);
        switch (type) {
	        case GNU:
		        switch (abi.getArchitecture()) {
			        case ARM:
			            return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("arm-linux-androideabi-g++" + executableSuffix));
			        case MIPS:
				        return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("mipsel-linux-android-g++" + executableSuffix));
			        case X86:
			        {
				        final AbsoluteFilePath compilerPath = new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-linux-android-g++" + executableSuffix));
				        if (compilerPath.exists()) {
					        return compilerPath;
				        } else {
					        /* Before Android NDK r8b x86 binaries had a different name */
					        return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-android-linux-g++" + executableSuffix));
				        }
			        }
			        default:
				        throw new Error(String.format("Unknown target ABI %s", abi.toString()));
		        }
	        default:
		        throw new Error(String.format("Unknown compiler type %s", type.toString()));
        }
    }

    private AbsoluteFilePath getCCompilerPath(AbsoluteDirectoryPath toolchainDirectory, ABI abi, Type type) {
	    final String executableSuffix = toolchainDirectory.getMachine().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : new String();
	    final AbsoluteDirectoryPath binariesDirectory = getBinariesDirectory(toolchainDirectory);
        switch (type) {
	        case GNU:
		        switch (abi.getArchitecture()) {
			        case ARM:
			            return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("arm-linux-androideabi-gcc" + executableSuffix));
			        case MIPS:
				        return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("mipsel-linux-android-gcc" + executableSuffix));
			        case X86:
			        {
				        final AbsoluteFilePath compilerPath = new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-linux-android-gcc" + executableSuffix));
				        if (compilerPath.exists()) {
					        return compilerPath;
				        } else {
					        /* Before Android NDK r8b x86 binaries had a different name */
					        return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-android-linux-gcc" + executableSuffix));
				        }
			        }
			        default:
				        throw new Error(String.format("Unknown target ABI %s", abi.toString()));
		        }
	        default:
		        throw new Error(String.format("Unknown compiler type %s", type.toString()));
        }
    }

	private AbsoluteFilePath getStripPath(AbsoluteDirectoryPath toolchainDirectory, ABI abi, Type type) {
		final String executableSuffix = toolchainDirectory.getMachine().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : new String();
		final AbsoluteDirectoryPath binariesDirectory = getBinariesDirectory(toolchainDirectory);
		switch (type) {
			case GNU:
				switch (abi.getArchitecture()) {
					case ARM:
						return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("arm-linux-androideabi-strip" + executableSuffix));
					case MIPS:
						return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("mipsel-linux-android-strip" + executableSuffix));
					case X86:
					{
						final AbsoluteFilePath compilerPath = new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-linux-android-strip" + executableSuffix));
						if (compilerPath.exists()) {
							return compilerPath;
						} else {
					        /* Before Android NDK r8b x86 binaries had a different name */
							return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-android-linux-strip" + executableSuffix));
						}
					}
					default:
						throw new Error(String.format("Unknown target ABI %s", abi.toString()));
				}
			default:
				throw new Error(String.format("Unknown compiler type %s", type.toString()));
		}
	}

	private AbsoluteFilePath getObjCopyPath(AbsoluteDirectoryPath toolchainDirectory, ABI abi, Type type) {
		final String executableSuffix = toolchainDirectory.getMachine().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : new String();
		final AbsoluteDirectoryPath binariesDirectory = getBinariesDirectory(toolchainDirectory);
		switch (type) {
			case GNU:
				switch (abi.getArchitecture()) {
					case ARM:
						return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("arm-linux-androideabi-objcopy" + executableSuffix));
					case MIPS:
						return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("mipsel-linux-android-objcopy" + executableSuffix));
					case X86:
					{
						final AbsoluteFilePath compilerPath = new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-linux-android-objcopy" + executableSuffix));
						if (compilerPath.exists()) {
							return compilerPath;
						} else {
					        /* Before Android NDK r8b x86 binaries had a different name */
							return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-android-linux-objcopy" + executableSuffix));
						}
					}
					default:
						throw new Error(String.format("Unknown target ABI %s", abi.toString()));
				}
			default:
				throw new Error(String.format("Unknown compiler type %s", type.toString()));
		}
	}

	private AbsoluteFilePath getAssemblerPath(AbsoluteDirectoryPath toolchainDirectory, ABI abi, Type type) {
	    final String executableSuffix = toolchainDirectory.getMachine().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : new String();
	    final AbsoluteDirectoryPath binariesDirectory = getBinariesDirectory(toolchainDirectory);
        switch (type) {
	        case GNU:
		        switch (abi.getArchitecture()) {
			        case ARM:
			            return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("arm-linux-androideabi-as" + executableSuffix));
			        case MIPS:
				        return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("mipsel-linux-android-as" + executableSuffix));
			        case X86:
			        {
				        final AbsoluteFilePath compilerPath = new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-linux-android-as" + executableSuffix));
				        if (compilerPath.exists()) {
					        return compilerPath;
				        } else {
					        /* Before Android NDK r8b x86 binaries had a different name */
					        return new AbsoluteFilePath(binariesDirectory, new RelativeFilePath("i686-android-linux-as" + executableSuffix));
				        }
			        }
			        default:
				        throw new Error(String.format("Unknown target ABI %s", abi.toString()));
		        }
	        default:
		        throw new Error(String.format("Unknown compiler type %s", type.toString()));
        }
    }

	public AndroidToolchain(AbsoluteDirectoryPath rootDirectory, ABI abi, Version version, Type type, AndroidPlatform platform) {
		this.abi = abi;
		this.type = type;
		this.version = version;
		this.platform = platform;
		if (type.equals(Type.GNU)) {
			final AbsoluteFilePath cppCompilerPath = getCppCompilerPath(rootDirectory, abi, type);
			final AbsoluteFilePath cCompilerPath = getCCompilerPath(rootDirectory, abi, type);
			final AbsoluteFilePath assemblerPath = getAssemblerPath(rootDirectory, abi, type);
			final AbsoluteFilePath stripPath = getStripPath(rootDirectory, abi, type);
			final AbsoluteFilePath objCopyPath = getObjCopyPath(rootDirectory, abi, type);
			final List<AbsoluteDirectoryPath> includeDirectories = new LinkedList<AbsoluteDirectoryPath>();
			includeDirectories.add(platform.getIncludeDirectory());
			final List<AbsoluteDirectoryPath> libraryDirectories = new LinkedList<AbsoluteDirectoryPath>();
			libraryDirectories.add(platform.getLibraryDirectory());
			this.cppCompiler = new GnuCppCompiler(cppCompilerPath, abi, version, includeDirectories, platform.getSysRoot());
			this.cCompiler = new GnuCCompiler(cCompilerPath, abi, version, includeDirectories, platform.getSysRoot());
			this.assembler = new GnuAssembler(assemblerPath, abi);
			this.linker = new GccCompatibleCompilerLinker(cCompilerPath, abi, version, libraryDirectories, platform.getSysRoot(), GnuCppCompiler.class);
			this.strip = new GnuStrip(stripPath, abi);
			this.objCopy = new GnuObjCopy(objCopyPath, abi);
		} else {
			throw new Error(String.format("Unsupported compiler type %s", type.toString()));
		}
	}

	@Override
	public final Version getVersion() {
		return this.version;
	}

	@Override
	public String toString() {
		return String.format("Android %s-based toolchain version %s for %s", getType().toString(), getVersion().toString(), getABI().toString());
	}

	public Type getType() {
		return this.type;
	}

	public ABI getABI() {
		return this.abi;
	}

	public CppCompiler getCppCompiler() {
		return this.cppCompiler.copy();
	}

	public CCompiler getCCompiler() {
		return this.cCompiler.copy();
	}

	public Assembler getAssembler() {
		return this.assembler.copy();
	}

	public Linker getLinker() {
		return this.linker.copy();
	}

	public GnuStrip getStrip() {
		return new GnuStrip(this.strip);
	}

	public GnuObjCopy getObjCopy() {
		return new GnuObjCopy(this.objCopy);
	}

	private final ABI abi;
	private final Version version;
	private final Type type;
	private AndroidPlatform platform;
	private final CppCompiler cppCompiler;
	private final CCompiler cCompiler;
	private final Assembler assembler;
	private final Linker linker;
	private final GnuStrip strip;
	private final GnuObjCopy objCopy;
}
