/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.generic.Versioned;
import info.yeppp.ebuilda.internals.Program;

public class AppleStrip implements Versioned {

	public static VersionList<AppleStrip> enumerate(Machine machine, ABI abi) {
		final VersionList<AppleStrip> strips = new VersionList<AppleStrip>();
		if (machine.getOperatingSystem().equals(OperatingSystem.MacOSX) && abi.getOperatingSystem().equals(OperatingSystem.MacOSX)) {
			final ToolInfo stripInfo = AppleStrip.verify(new AbsoluteFilePath(machine, "/usr/bin/strip"));
			if (stripInfo.isValid()) {
				final AppleStrip strip = new AppleStrip(stripInfo.getPath(), abi, stripInfo.getVersion());
				strips.add(strip);
			}
		}
		return strips;
	}

	public static ToolInfo verify(AbsoluteFilePath stripPath) {
		if (stripPath.getCanonicalForm().equals("/usr/bin/strip") && stripPath.exists()) {
			return new ToolInfo(stripPath, stripPath.getMachine().getArchitecture().is64bit() ? ABI.X64_MacOSX_SystemV_Default : ABI.X86_MacOSX_Pic_Default,
				new ABI[]{ABI.X64_MacOSX_SystemV_Default, ABI.X86_MacOSX_Pic_Default}, new Version(0));
		} else {
			return new ToolInfo(stripPath);
		}
	}

	public AppleStrip(AbsoluteFilePath stripPath, ABI abi, Version version) {
		this.stripPath = stripPath;
		this.abi = abi;
		this.version = version;
		this.verboseBuild = false;
	}

	public AppleStrip(AppleStrip prototype) {
		this.stripPath = prototype.stripPath;
		this.abi = prototype.abi;
		this.version = prototype.version;
		this.verboseBuild = prototype.verboseBuild;
	}

	public BuildMessages stripLocalSymbols(AbsoluteFilePath binary) {
		final BuildMessages buildMessages = new BuildMessages();
		final Program.Parameters parameters = new Program.Parameters();
		parameters.add("-x");
		parameters.add(binary.getCanonicalForm());
		final Program.Environment environment = new Program.Environment();
		final Program.EventHandler eventHandler = new PassThroughOutputParser(buildMessages, binary);
		if (isVerboseBuild()) {
			System.out.println(stripPath + " " + parameters.toString());
		}
		final Program strip = new Program(stripPath, parameters, environment, eventHandler);
		strip.join();
		return buildMessages;
	}

	public void setVerboseBuild(boolean verboseBuild) {
		this.verboseBuild = verboseBuild;
	}

	public boolean isVerboseBuild() {
		return this.verboseBuild;
	}

	public ABI getABI() {
		return abi;
	}

	@Override
	public Version getVersion() {
		return version;
	}

	private final AbsoluteFilePath stripPath;
	private final ABI abi;
	private final Version version;
	private boolean verboseBuild;
}
