/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.generic.Assembler;
import info.yeppp.ebuilda.internals.Program;

import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class GnuAssembler extends AbstractAssembler {

	/* Examples of first output line:
	 * "GNU assembler (GNU Binutils for Ubuntu) 2.23.2"
	 * "GNU assembler (GNU Binutils for Debian) 2.22"
	 * "GNU assembler (GNU Binutils) 2.22.90.20120727"
	 * "GNU assembler (GNU Binutils; SUSE Linux Enterprise PTF) 2.21.1"
	 * "GNU assembler (GNU Binutils) 2.20.1.20100303 20121001 (Native Client r9900, Git Commit e0648d331e12a691e41645592005e72ace44667b)
	 * "GNU assembler version 2.22.52.0.1-10.fc17 20120131"
	 * "GNU assembler version 2.20.51.0.2-5.36.el6 20100205"
	 */
	private static final Pattern debianVersionPattern = Pattern.compile("GNU assembler \\(.+\\) (\\d+(?:\\.\\d+)+)\\b\\.*");
	private static final Pattern redhatVersionPattern = Pattern.compile("GNU assembler version (\\d+(?:\\.\\d+)+)\\-.+");

	/* Examples of last output line:
	 * "This assembler was configured for a target of `x86_64-linux-gnu'."
	 * "This assembler was configured for a target of `x86_64-redhat-linux'."
	 * "This assembler was configured for a target of `x86_64-suse-linux'."
	 * "This assembler was configured for a target of `x86_64-k1om-linux'."
	 * "This assembler was configured for a target of `mingw32'."
	 * "This assembler was configured for a target of `i686-w64-mingw32'."
	 * "This assembler was configured for a target of `arm-linux-gnueabi'."
	 * "This assembler was configured for a target of `arm-linux-gnueabihf'."
	 * "This assembler was configured for a target of `arm-linux-androideabi'."
	 * "This assembler was configured for a target of `mipsel-linux-android'."
	 * "This assembler was configured for a target of `i686-linux-android'."
	 * "This assembler was configured for a target of `i686-android-linux'."
	 * "This assembler was configured for a target of `x86_64-nacl'."
	 * "This assembler was configured for a target of `arm-nacl'."
	 */
	private static final Pattern targetRegex = Pattern.compile("This assembler was configured for a target of `(\\w+(?:\\-\\w+)*)'.");

	public static ToolInfo verify(AbsoluteFilePath assemblerPath) {
		Program.Parameters parameters = new Program.Parameters("-version");
		Program.Environment environment = new Program.Environment();
		final List<String> outputStrings = Program.getStdoutLines(assemblerPath, parameters, environment, 6);
		if (outputStrings.size() < 6) {
			return new ToolInfo(assemblerPath);
		} else {
			final String versionString = outputStrings.get(0);
			final Matcher versionMatcher = debianVersionPattern.matcher(versionString).matches() ?
					debianVersionPattern.matcher(versionString) : redhatVersionPattern.matcher(versionString);
			final String targetString = outputStrings.get(5);
			final Matcher targetMatcher = targetRegex.matcher(targetString);
			ABI defaultABI = null;
			ABI[] supportedABI = null;
			if (versionMatcher.matches() && targetMatcher.matches()) {
				final String target = targetMatcher.group(1);
				if (target.equals("x86_64-k1om-linux")) {
					defaultABI = ABI.X64_Linux_KNC_Default;
					supportedABI = new ABI[]{ ABI.X64_Linux_KNC_Default };
				} else if (target.matches("x86_64\\-(\\w+\\-)?linux(\\-gnu)?")) {
					defaultABI = ABI.X64_Linux_SystemV_Default;
					supportedABI = new ABI[]{ ABI.X64_Linux_SystemV_Default, ABI.X86_Linux_Pic_i586 };
				} else if (target.equals("arm-linux-gnueabihf")) {
					defaultABI = ABI.ARM_Linux_HardEABI_V7A;
					supportedABI = new ABI[]{ ABI.ARM_Linux_HardEABI_V7A };
				} else if (target.equals("arm-linux-gnueabi")) {
					defaultABI = ABI.ARM_Linux_SoftEABI_V5T;
					supportedABI = new ABI[]{ ABI.ARM_Linux_SoftEABI_V5T };
				} else if (target.equals("arm-linux-androideabi")) {
					defaultABI = ABI.ARM_Linux_SoftEABI_Android;
					supportedABI = new ABI[]{ ABI.ARM_Linux_SoftEABI_Android, ABI.ARM_Linux_SoftEABI_AndroidV7A };
				} else if (target.equals("i686-linux-android") || target.equals("i686-android-linux")) {
					defaultABI = ABI.X86_Linux_Pic_Android;
					supportedABI = new ABI[]{ ABI.X86_Linux_Pic_Android };
				} else if (target.equals("mipsel-linux-android")) {
					defaultABI = ABI.MIPS_Linux_O32_Android;
					supportedABI = new ABI[]{ ABI.MIPS_Linux_O32_Android };
				} else if (target.equals("arm-nacl")) {
					defaultABI = ABI.ARM_NaCl_NaCl_Default;
					supportedABI = new ABI[]{ ABI.ARM_NaCl_NaCl_Default };
				} else if (target.equals("x86_64-nacl")) {
					defaultABI = ABI.X64_NaCl_NaCl_Default;
					supportedABI = new ABI[]{ ABI.X64_NaCl_NaCl_Default, ABI.X86_NaCl_NaCl_i686 };
				}
				final Version version = new Version(versionMatcher.group(1));
				return new ToolInfo(assemblerPath, defaultABI, supportedABI, version);
			} else {
				return new ToolInfo(assemblerPath);
			}
		}
	}

	public GnuAssembler(AbsoluteFilePath assemblerPath, ABI abi) {
		super(abi);
		this.assemblerPath = assemblerPath;
	}

	public GnuAssembler(GnuAssembler prototype) {
		super(prototype);
		this.assemblerPath = prototype.assemblerPath;
	}

	public BuildMessages assemble(AbsoluteFilePath source) {
		final AbsoluteFilePath object = getObjectPath(source);

		if (!object.getDirectory().create()) {
			object.tryDelete();
		}

		final BuildMessages buildMessages = new BuildMessages();
		final Program.Parameters parameters = prepareParameters(source, object);
		final Program.Environment environment = new Program.Environment();
		final Program.EventHandler eventHandler = new PassThroughOutputParser(buildMessages, source);
		if (isVerboseBuild()) {
			System.out.println(assemblerPath.toString() + " " + parameters.toString());
		}
		final Program assembler = new Program(assemblerPath, parameters, environment, eventHandler);
		assembler.join();
		return buildMessages;
	}

	protected Program.Parameters prepareParameters(AbsoluteFilePath sourcePath, AbsoluteFilePath objectPath) {
		final Program.Parameters parameters = new Program.Parameters();
		parameters.add("--noexecstack");
		for (final Map.Entry<String, String> macro : macros.entrySet()) {
			parameters.add("--defsym", macro.getKey() + "=" + macro.getValue());
		}
		parameters.add(sourcePath.getCanonicalForm());
		parameters.add("-o", objectPath.getCanonicalForm());
		return parameters;
	}

	@Override
	public Assembler copy() {
		return new GnuAssembler(this);
	}

	private final AbsoluteFilePath assemblerPath;
}
