/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.generic.CCompiler;
import info.yeppp.ebuilda.generic.CppCompiler;
import info.yeppp.ebuilda.generic.Versioned;
import info.yeppp.ebuilda.internals.Program;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class IntelCCompiler extends AbstractCCompiler implements Versioned {

	private static final Pattern versionPattern = Pattern.compile("icc version (\\d+(?:\\.\\d+)+)\\b.*");

	public static ToolInfo verify(AbsoluteFilePath compilerPath) {
		Program.Parameters parameters = new Program.Parameters("-v");
		Program.Environment environment = new Program.Environment();
		final List<String> outputStrings = Program.getStderrLines(compilerPath, parameters, environment, 1);
		if (outputStrings.isEmpty()) {
			return new ToolInfo(compilerPath);
		} else {
			final String versionString = outputStrings.get(0);
			final Matcher versionMatcher = versionPattern.matcher(versionString);
			if (versionMatcher.matches()) {
				final Version version = new Version(versionMatcher.group(1));
				final ABI[] supportedABI = version.compareTo(new Version(13)) >= 0 ?
						new ABI[]{ ABI.X86_Linux_Pic_i586, ABI.X64_Linux_SystemV_Default, ABI.X64_Linux_KNC_Default } :
						new ABI[]{ ABI.X86_Linux_Pic_i586, ABI.X64_Linux_SystemV_Default };
				final String binSubDirectory = compilerPath.getDirectory().getLastFileName();
				ABI defaultABI = ABI.X86_Linux_Pic_i586;
				if (binSubDirectory.equals("intel64")) {
					defaultABI = ABI.X64_Linux_SystemV_Default;
				} else if (binSubDirectory.equals("intel64_mic")) {
					defaultABI = ABI.X64_Linux_KNC_Default;
				} else if (binSubDirectory.equals("bin")) {
					defaultABI = compilerPath.getMachine().getDefaultABI();
				}
				return new ToolInfo(compilerPath, defaultABI, supportedABI, version);
			} else {
				return new ToolInfo(compilerPath);
			}
		}
	}

	IntelCCompiler(AbsoluteFilePath compilerPath, ABI abi, Version version, Collection<AbsoluteDirectoryPath> defaultIncludeDirectories, AbsoluteDirectoryPath sysRoot) {
		super(new IntelCppCompiler(compilerPath, abi, version, defaultIncludeDirectories, sysRoot));
	}

	public IntelCCompiler(IntelCCompiler prototype) {
		super(prototype);
	}

	public GccCompatibleCompilerLinker asLinker(Collection<AbsoluteDirectoryPath> defaultGlobalLibraryDirectories) {
		return ((IntelCppCompiler)this.cppCompiler).asLinker(defaultGlobalLibraryDirectories);
	}

	public BuildMessages compile(AbsoluteFilePath source) {
		return this.cppCompiler.compile(source);
	}

	@Override
	public final void addDefaultGlobalIncludeDirectories() {
		this.cppCompiler.addDefaultGlobalIncludeDirectories();
	}

	@Override
	public CCompiler copy() {
		return new IntelCCompiler(this);
	}

	@Override
	public final Version getVersion() {
		return ((IntelCppCompiler)this.cppCompiler).getVersion();
	}

}
