/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.generic.CppCompiler;
import info.yeppp.ebuilda.generic.Versioned;
import info.yeppp.ebuilda.internals.Program;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class IntelCppCompiler extends AbstractCppCompiler implements Versioned {

	private static final Pattern versionPattern = Pattern.compile("icpc version (\\d+(?:\\.\\d+)+)\\b.*");

	public static ToolInfo verify(AbsoluteFilePath compilerPath) {
		Program.Parameters parameters = new Program.Parameters("-v");
		Program.Environment environment = new Program.Environment();
		final List<String> outputStrings = Program.getStderrLines(compilerPath, parameters, environment, 1);
		if (outputStrings.isEmpty()) {
			return new ToolInfo(compilerPath);
		} else {
			final String versionString = outputStrings.get(0);
			final Matcher versionMatcher = versionPattern.matcher(versionString);
			if (versionMatcher.matches()) {
				final Version version = new Version(versionMatcher.group(1));
				final ABI[] supportedABI = version.compareTo(new Version(13)) >= 0 ?
						new ABI[]{ ABI.X86_Linux_Pic_i586, ABI.X64_Linux_SystemV_Default, ABI.X64_Linux_KNC_Default } :
						new ABI[]{ ABI.X86_Linux_Pic_i586, ABI.X64_Linux_SystemV_Default };
				final String binSubDirectory = compilerPath.getDirectory().getLastFileName();
				ABI defaultABI = ABI.X86_Linux_Pic_i586;
				if (binSubDirectory.equals("intel64")) {
					defaultABI = ABI.X64_Linux_SystemV_Default;
				} else if (binSubDirectory.equals("intel64_mic")) {
					defaultABI = ABI.X64_Linux_KNC_Default;
				} else if (binSubDirectory.equals("bin")) {
					defaultABI = compilerPath.getMachine().getDefaultABI();
				}
				return new ToolInfo(compilerPath, defaultABI, supportedABI, version);
			} else {
				return new ToolInfo(compilerPath);
			}
		}
	}

	IntelCppCompiler(AbsoluteFilePath compilerPath, ABI abi, Version version, Collection<AbsoluteDirectoryPath> defaultIncludeDirectories, AbsoluteDirectoryPath sysRoot) {
		super(abi);
		this.compilerPath = compilerPath;
		this.defaultIncludeDirectories = new ArrayList<AbsoluteDirectoryPath>(defaultIncludeDirectories);
		this.sysRoot = sysRoot;
		this.version = version;
	}

	public IntelCppCompiler(IntelCppCompiler prototype) {
		super(prototype);
		this.compilerPath = prototype.compilerPath;
		this.defaultIncludeDirectories = new ArrayList<AbsoluteDirectoryPath>(prototype.defaultIncludeDirectories);
		this.version = prototype.version;
		this.sysRoot = prototype.sysRoot;
		this.pic = prototype.pic;
	}

	public GccCompatibleCompilerLinker asLinker(Collection<AbsoluteDirectoryPath> defaultGlobalLibraryDirectories) {
		return new GccCompatibleCompilerLinker(this.compilerPath, getABI(), getVersion(), defaultGlobalLibraryDirectories, getSysRoot(), IntelCppCompiler.class);
	}

	public BuildMessages compile(AbsoluteFilePath source) {
		final BuildMessages buildMessages = new BuildMessages();
		final AbsoluteFilePath object = getObjectPath(source);

		if (!object.getDirectory().create()) {
			object.tryDelete();
		}

		final Program.Parameters parameters = prepareParameters(source, object);

		final Program.EventHandler eventHandler = new GccOutputParser(buildMessages, source, isVerboseBuild());
		if (isVerboseBuild()) {
			System.out.println(compilerPath + " " + parameters.toString());
		}
		final Program compiler = new Program(compilerPath, parameters, new Program.Environment(), eventHandler);
		compiler.join();
		return buildMessages;
	}

	protected final Program.Parameters prepareParameters(AbsoluteFilePath sourcePath, AbsoluteFilePath objectPath) {
		final Program.Parameters parameters = new Program.Parameters();
		parameters.add("-c");
		if (getDebugInformation()) {
			parameters.add("-g");
		}
		switch (pic) {
			case NoPIC:
				parameters.add("-fno-pic");
				break;
			case LimitedLibraryPIC:
				parameters.add("-fpic");
				break;
			case UnlimitedLibraryPIC:
				parameters.add("-fPIC");
				break;
			case LimitedExecutablePIC:
				parameters.add("-fpie");
				break;
			case UnlimitedExecutablePIC:
				parameters.add("-fPIE");
				break;
		}
		if (getSysRoot() != null) {
			parameters.add("--sysroot=" + getSysRoot().getCanonicalForm());
		}
		switch (getOptimization()) {
			case NoOptimization:
				parameters.add("-O0");
				break;
			case MinSizeOptimization:
				parameters.add("-Os");
				break;
			case SpeedOptimization:
				parameters.add("-O2");
				break;
			case MaxSpeedOptimization:
				parameters.add("-O3");
				break;
		}
		if (getGlobalOptimization()) {
			parameters.add("-ipo");
		}
		for (Map.Entry<String, String> macro: macros.entrySet()) {
			parameters.add("-D" + macro.getKey() + "=" + macro.getValue());
		}
		parameters.add("-Wa,--noexecstack");
		switch (runtimeLibrary) {
			case NoRuntimeLibrary:
				parameters.add("-nostdlib", "-fno-stack-protector"); break;
			case DynamicRuntimeLibrary:
				if (!getABI().getOperatingSystem().equals(OperatingSystem.MacOSX)) {
					parameters.add("-shared-intel");
				}
				break;
			case StaticRuntimeLibrary:
				if (!getABI().getOperatingSystem().equals(OperatingSystem.MacOSX)) {
					parameters.add("-static-intel");
				}
				break;
		}
		if (rttiEnabled != null) {
			parameters.add(rttiEnabled ? "-frtti" : "-fno-rtti");
		}
		if (exceptions != null) {
			switch (exceptions) {
				case NoExceptions:
					parameters.add("-fno-exceptions", "-fno-asynchronous-unwind-tables");
					break;
				case DisabledExceptions:
					parameters.add("-fno-exceptions", "-fno-asynchronous-unwind-tables");
					break;
				case SyncronousExceptions:
					parameters.add("-fexceptions", "-fno-asynchronous-unwind-tables");
					break;
				case AsyncronousExceptions:
					parameters.add("-fexceptions", "-fasynchronous-unwind-tables");
					break;
			}
		}
		switch (getABI()) {
			case X64_Linux_SystemV_Default:
				parameters.add("-m64");
				break;
			case X86_Linux_Pic_i586:
				parameters.add("-m32");
				break;
			case X64_Linux_KNC_Default:
				parameters.add("-mmic");
				break;
			default:
				throw new Error(String.format("Unsupported ABI %s", getABI().toString()));
		}
		if (getABI().getOperatingSystem().equals(OperatingSystem.Linux)) {
			parameters.add("-ffunction-sections", "-fdata-sections");
		}
		parameters.add("-fomit-frame-pointer", "-fstrict-aliasing");
		for (final AbsoluteDirectoryPath includeDirectory : includeDirectories) {
			parameters.add("-I" + includeDirectory.getCanonicalForm());
		}
		for (final AbsoluteDirectoryPath globalIncludeDirectory : globalIncludeDirectories) {
			parameters.add("-I" + globalIncludeDirectory.getCanonicalForm());
		}
		parameters.add(sourcePath.getCanonicalForm());
		parameters.add("-o", objectPath.getCanonicalForm());
		return parameters;
	}

	@Override
	public final void addDefaultGlobalIncludeDirectories() {
		addGlobalIncludeDirectories(defaultIncludeDirectories);
	}

	@Override
	public CppCompiler copy() {
		return new IntelCppCompiler(this);
	}

	@Override
	public final Version getVersion() {
		return this.version;
	}

	private final AbsoluteFilePath compilerPath;
	private final List<AbsoluteDirectoryPath> defaultIncludeDirectories;
	private final Version version;
}
