/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.Versioned;
import info.yeppp.ebuilda.internals.WinRegistry;

import java.util.*;
import java.util.regex.Pattern;

public class JavaSDK implements Versioned {

	private static List<ToolsetInfo> initLocalBinaries() {
		final String executableExtension = Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : "";
		final List<ToolsetInfo> toolsetCandidates = new ArrayList<ToolsetInfo>();
		final HashSet<AbsoluteDirectoryPath> candidateDirectories = new HashSet<AbsoluteDirectoryPath>(Machine.getLocal().getStandardBinaryDirectories());
		if (Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows)) {
			try {
				final String installPath = WinRegistry.readString(WinRegistry.HKEY_CURRENT_USER, "SOFTWARE\\JavaSoft\\Java Development Kit\\1.6", "");
				if (installPath != null) {
					final AbsoluteDirectoryPath installDirectory = new AbsoluteDirectoryPath(Machine.getLocal(), installPath);
					candidateDirectories.add(installDirectory);
				}
			} catch (Exception e) {
			}
			try {
				final String installPath = WinRegistry.readString(WinRegistry.HKEY_CURRENT_USER, "SOFTWARE\\JavaSoft\\Java Development Kit\\1.7", "");
				if (installPath != null) {
					final AbsoluteDirectoryPath installDirectory = new AbsoluteDirectoryPath(Machine.getLocal(), installPath);
					candidateDirectories.add(installDirectory);
				}
			} catch (Exception e) {
			}
		}
		for (final AbsoluteDirectoryPath path : candidateDirectories) {
			final RelativeFilePath javacName = new RelativeFilePath("javac" + executableExtension);
			final AbsoluteFilePath javacPath = new AbsoluteFilePath(path, javacName);
			if (javacPath.exists()) {
				try {
					final AbsoluteDirectoryPath jdkPath = javacPath.resolve().getDirectory().getParent();
					if (jdkPath.exists() && JavaSDK.getBinariesDirectory(jdkPath).exists())
					toolsetCandidates.add(new ToolsetInfo(jdkPath, null, null));
				} catch (Exception e) {
				}
			}
		}
		final List<ToolsetInfo> toolsets = new ArrayList<ToolsetInfo>(toolsetCandidates.size());
		for (final ToolsetInfo toolsetCandidate : toolsetCandidates) {
			final RelativeFilePath javacName = new RelativeFilePath("javac" + executableExtension);
			final AbsoluteFilePath javacPath = new AbsoluteFilePath(JavaSDK.getBinariesDirectory(toolsetCandidate.getPath()), javacName);
			final ToolInfo javacInfo = JavacJavaCompiler.verify(javacPath);
			if (javacInfo.isValid()) {
				final Version version = javacInfo.getVersion();
				toolsets.add(new ToolsetInfo(toolsetCandidate.getPath(), null, null, null, null, version));
			}
		}
		return toolsets;
	}

	private static List<ToolsetInfo> localBinaries = JavaSDK.initLocalBinaries();

	public static VersionList<JavaSDK> enumerate(Machine machine) {
		final VersionList<JavaSDK> toolchains = new VersionList<JavaSDK>();
		for (final ToolsetInfo toolsetInfo : localBinaries) {
			final JavaSDK toolchain = new JavaSDK(toolsetInfo.getPath(), toolsetInfo.getVersion());
			toolchains.add(toolchain);
		}
		return toolchains;
	}

	JavaSDK(AbsoluteDirectoryPath path, Version version) {
		this.path = path;
		this.version = version;
	}

	public AbsoluteDirectoryPath getBinariesDirectory() {
		return JavaSDK.getBinariesDirectory(this.path);
	}

	protected static AbsoluteDirectoryPath getBinariesDirectory(AbsoluteDirectoryPath jdkPath) {
		switch (jdkPath.getMachine().getOperatingSystem()) {
			case Windows:
			case Linux:
				return new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("bin"));
			case MacOSX:
				return new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("Commands"));
			default:
				return null;
		}
	}

	public Collection<AbsoluteDirectoryPath> getIncludeDirectories() {
		return JavaSDK.getIncludeDirectories(this.path);
	}

	protected static Collection<AbsoluteDirectoryPath> getIncludeDirectories(AbsoluteDirectoryPath jdkPath) {
		final List<AbsoluteDirectoryPath> includeDirectories = new ArrayList<AbsoluteDirectoryPath>(2);
		switch (jdkPath.getMachine().getOperatingSystem()) {
			case Windows:
				includeDirectories.add(new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("include")));
				includeDirectories.add(new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("include/win32")));
				break;
			case Linux:
				includeDirectories.add(new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("include")));
				includeDirectories.add(new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("include/linux")));
				break;
			case MacOSX:
				includeDirectories.add(new AbsoluteDirectoryPath(jdkPath, new RelativeDirectoryPath("Headers")));
				break;
		}
		return includeDirectories;
	}

	@Override
	public Version getVersion() {
		return this.version;
	}

	@Override
	public String toString() {
		return String.format("Java SDK version %s", getVersion().toString());
	}

	protected final AbsoluteDirectoryPath path;
	protected final Version version;
}
