/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.PathFormatException;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.*;

public class Machine {
	private static final Machine localMachine = new Machine();

	public static final Machine getLocal() {
		return Machine.localMachine;
	}

	private Machine() {
		this.operatingSystem = OperatingSystem.detect();
		this.architecture = Architecture.detect();
	}

	public OperatingSystem getOperatingSystem() {
		return this.operatingSystem;
	}

	public Architecture getArchitecture() {
		return this.architecture;
	}

	public ABI getDefaultABI() {
		switch (getOperatingSystem()) {
			case Linux:
				switch (getArchitecture()) {
					case X86:
						return ABI.X86_Linux_Pic_i586;
					case X64:
						return ABI.X64_Linux_SystemV_Default;
					case ARM:
						return ABI.ARM_Linux_HardEABI_V7A;
					default:
						return null;
				}
			case Windows:
				switch (getArchitecture()) {
					case X86:
						return ABI.X86_Windows_Default_i586;
					case X64:
						return ABI.X64_Windows_Microsoft_Default;
					default:
						return null;
				}
			case MacOSX:
				switch (getArchitecture()) {
					case X86:
						return ABI.X86_MacOSX_Pic_Default;
					case X64:
						return ABI.X64_MacOSX_SystemV_Default;
					default:
						return null;
				}
			default:
				return null;
		}
	}

	public boolean isLocal() {
		return this == localMachine;
	}

	public AbsoluteDirectoryPath getWorkingDirectory() {
		if (isLocal()) {
			return new AbsoluteDirectoryPath(this, System.getProperty("user.dir"));
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	public AbsoluteDirectoryPath getUserHomeDirectory() {
		if (isLocal()) {
			return new AbsoluteDirectoryPath(this, System.getProperty("user.home"));
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	public AbsoluteDirectoryPath getApplicationDirectory() {
		if (isLocal()) {
			final URL url = this.getClass().getProtectionDomain().getCodeSource().getLocation();
			try {
				final String path = URLDecoder.decode(url.getPath(), "UTF-8");
				final File file = new File(path);
				return new AbsoluteDirectoryPath(this, file.getAbsolutePath());
			} catch (UnsupportedEncodingException e) {
				throw new Error("Unsupported JavaVM: lacks UTF-8 support");
			}
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	public List<AbsoluteDirectoryPath> getPathVariable(String name) {
		if (isLocal()) {
			final String pathVariable = getVariable(name);
			final List<AbsoluteDirectoryPath> paths = new LinkedList<AbsoluteDirectoryPath>();
			final String pathSeparator = System.getProperty("path.separator");
			int startIndex = -1;
			do {
				int endIndex = pathVariable.indexOf(pathSeparator, startIndex + 1);
				final String path = pathVariable.substring(startIndex + 1, (endIndex == -1) ? pathVariable.length() : endIndex);
				try {
					paths.add(new AbsoluteDirectoryPath(this, path));
				} catch (Exception e) {
				}
				startIndex = endIndex;
			} while (startIndex != -1);
			return paths;
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	private static String getLocalVariable(String name) {
		for (final Map.Entry<String, String> variable : System.getenv().entrySet()) {
			if (variable.getKey().equalsIgnoreCase(name)) {
				return variable.getValue();
			}
		}
		return new String();
	}

	public String getVariable(String name) {
		if (isLocal()) {
			return Machine.getLocalVariable(name);
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	private static Set<AbsoluteDirectoryPath> initStandardBinaryDirectories() {
		final HashSet<AbsoluteDirectoryPath> directories = new HashSet<AbsoluteDirectoryPath>();
		final String path = getLocalVariable("PATH");
		final int pathSeparatorLength = pathSeparator.length();
		int start = -pathSeparatorLength;
		int end = path.indexOf(pathSeparator, start + pathSeparatorLength);
		while (end != -1) {
			final String directory = path.substring(start + pathSeparatorLength, end);
			try {
				directories.add(new AbsoluteDirectoryPath(Machine.getLocal(), directory));
			} catch (PathFormatException e) {
			}
			start = end;
			end = path.indexOf(pathSeparator, start + pathSeparatorLength);
		}
		final String directory = path.substring(start + pathSeparatorLength);
		directories.add(new AbsoluteDirectoryPath(Machine.getLocal(), directory));
		if (Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Linux)) {
			directories.add(new AbsoluteDirectoryPath(Machine.getLocal(), "/bin/"));
			directories.add(new AbsoluteDirectoryPath(Machine.getLocal(), "/usr/bin/"));
		}
		return directories;
	}

	public Set<AbsoluteDirectoryPath> getStandardBinaryDirectories() {
		if (isLocal()) {
			return Machine.standardBinaryDirectories;
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	private static final String pathSeparator = System.getProperty("path.separator");
	private static final Set<AbsoluteDirectoryPath> standardBinaryDirectories = Collections.unmodifiableSet(Machine.initStandardBinaryDirectories());
	private final OperatingSystem operatingSystem;
	private final Architecture architecture;
}
