/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.generic.CppCompiler;
import info.yeppp.ebuilda.internals.Program;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MicrosoftCppCompiler extends AbstractCppCompiler {

	private static Pattern versionRegex = Pattern.compile("Microsoft \\(R\\) (?:32\\-bit )?C/C\\+\\+ Optimizing Compiler Version (\\d+(?:\\.\\d+)+) for (\\w+)");

	public static ToolInfo verify(AbsoluteFilePath compilerPath, Collection<AbsoluteDirectoryPath> additionalPathDirectories) {
		Program.Parameters parameters = new Program.Parameters();
		Program.Environment environment = new Program.Environment();

		for (final AbsoluteDirectoryPath additionalPathDirectory : additionalPathDirectories) {
			environment.appendPathVariable("PATH", additionalPathDirectory.getCanonicalForm());
		}
		final List<String> outputStrings = Program.getStderrLines(compilerPath, parameters, environment, 1);
		if (outputStrings.isEmpty()) {
			return new ToolInfo(compilerPath);
		} else {
			final String versionString = outputStrings.get(0);
			final Matcher versionMatcher = versionRegex.matcher(versionString);
            if (versionMatcher.matches()) {
				final String targetArchitecture = versionMatcher.group(2);
	            ABI defaultABI = null;
	            if (targetArchitecture.equals("80x86") || targetArchitecture.equals("x86")) {
		            defaultABI = ABI.X86_Windows_Default_i586;
	            } else if (targetArchitecture.equals("x64")) {
		            defaultABI = ABI.X64_Windows_Microsoft_Default;
	            } else if (targetArchitecture.equals("ARM")) {
		            defaultABI = ABI.ARM_Windows_HardEABI_Default;
	            } else if (targetArchitecture.equals("Itanium")) {
		            defaultABI = ABI.IA64_Windows_Microsoft_Default;
	            }
				final Version version = new Version(versionMatcher.group(1));
	            if (defaultABI != null) {
	                return new ToolInfo(compilerPath, defaultABI, new ABI[]{defaultABI}, version);
	            } else {
		            return new ToolInfo(compilerPath);
				}
			} else {
				return new ToolInfo(compilerPath);
			}
		}
	}

	MicrosoftCppCompiler(AbsoluteFilePath compilerPath, ABI abi, Version version, Collection<AbsoluteDirectoryPath> additionalPathDirectories, Collection<AbsoluteDirectoryPath> defaultIncludeDirectories) {
		super(abi);
		this.compilerPath = compilerPath;
		this.version = version;
		this.additionalPathDirectories = new ArrayList<AbsoluteDirectoryPath>(additionalPathDirectories);
		this.defaultIncludeDirectories = new ArrayList<AbsoluteDirectoryPath>(defaultIncludeDirectories);
	}

	protected MicrosoftCppCompiler(MicrosoftCppCompiler prototype) {
		super(prototype);
		this.compilerPath = prototype.compilerPath;
		this.additionalPathDirectories = new ArrayList<AbsoluteDirectoryPath>(prototype.additionalPathDirectories);
		this.defaultIncludeDirectories = new ArrayList<AbsoluteDirectoryPath>(prototype.defaultIncludeDirectories);
		this.version = prototype.version;
	}

	public BuildMessages compile(AbsoluteFilePath source) {
		final BuildMessages buildMessages = new BuildMessages();
		final AbsoluteFilePath object = getObjectPath(source);
		if (!object.getDirectory().create()) {
			object.tryDelete();
		}

		final Program.Parameters parameters = prepareParameters(source, object);
		final Program.Environment environment = prepareEnvironment();

		final Program.EventHandler eventHandler = new MicrosoftCppCompilerOutputParser(buildMessages, source, isVerboseBuild());
		if (isVerboseBuild()) {
			System.out.println(compilerPath + " " + parameters.toString());
		}
		final Program compiler = new Program(compilerPath, parameters, environment, eventHandler);
		compiler.join();
		return buildMessages;
	}

	protected Program.Parameters prepareParameters(AbsoluteFilePath sourcePath, AbsoluteFilePath objectPath) {
		final Program.Parameters parameters = new Program.Parameters();
		parameters.add("/nologo");
		parameters.add("/c");
		parameters.add("/Fo" + objectPath.getCanonicalForm());
		parameters.add("/GS-");
		if (getDebugInformation()) {
			parameters.add("/Z7");
		}
		parameters.add(getGlobalOptimization() ? "/GL" : "/GL-");
		for (final AbsoluteDirectoryPath includeDirectory : this.includeDirectories) {
			parameters.add("/I" + includeDirectory);
		}
		switch (getOptimization()) {
			case NoOptimization:
				parameters.add("/Od"); break;
			case MinSizeOptimization:
				parameters.add("/O1"); break;
			case SpeedOptimization:
			case MaxSpeedOptimization:
				parameters.add("/O2"); break;
		}
		for (Map.Entry<String, String> macro : macros.entrySet()) {
			parameters.add("/D" + macro.getKey() + "=" + macro.getValue());
		}
		switch (getRuntimeLibrary()) {
			case NoRuntimeLibrary:
				parameters.add("/Zl"); break;
			case DynamicRuntimeLibrary:
				parameters.add(isDebugBuild() ? "/MDd" : "/MD"); break;
			case StaticRuntimeLibrary:
				parameters.add(isDebugBuild() ? "/MTd" : "/MT"); break;
		}
		if (isRttiEnabled() != null) {
			parameters.add(isRttiEnabled() ? "/GR" : "/GR-");
		}
		if (getABI().equals(ABI.X86_Windows_Default_i586)) {
			parameters.add("/arch:IA32");
		}
		if (getExceptionsEnabled() != null) {
			switch (getExceptionsEnabled()) {
				case NoExceptions:
				case DisabledExceptions:
					parameters.add("/EHs-"); break;
				case SyncronousExceptions:
					parameters.add("/EHsc"); break;
				case AsyncronousExceptions:
					parameters.add("/EHa"); break;
			}
		}
		parameters.add(sourcePath.getCanonicalForm());
		return parameters;
	}

	private Program.Environment prepareEnvironment() {
		final Program.Environment environment = new Program.Environment();
		for (final AbsoluteDirectoryPath additionalPathDirectory : this.additionalPathDirectories) {
			environment.prependPathVariable("PATH", additionalPathDirectory.getCanonicalForm());
		}
		for (final AbsoluteDirectoryPath globalIncludeDirectory : globalIncludeDirectories) {
			environment.prependPathVariable("INCLUDE", globalIncludeDirectory.getCanonicalForm());
		}
		return environment;
	}

	@Override
	public void addDefaultGlobalIncludeDirectories() {
		this.addGlobalIncludeDirectories(this.defaultIncludeDirectories);
	}

	@Override
	public CppCompiler copy() {
		return new MicrosoftCppCompiler(this);
	}

	public Version getVersion() {
		return this.version;
	}

	private final AbsoluteFilePath compilerPath;
	private final List<AbsoluteDirectoryPath> additionalPathDirectories;
	private final List<AbsoluteDirectoryPath> defaultIncludeDirectories;
	private final Version version;

}
