package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.internals.Program;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MicrosoftCppCompilerOutputParser implements Program.EventHandler {

	MicrosoftCppCompilerOutputParser(BuildMessages buildMessages, AbsoluteFilePath source, boolean verboseBuild) {
		this.buildMessages = buildMessages;
		this.verboseBuild = verboseBuild;
		this.source = source;
		this.filename = source.getRelativePath(source.getDirectory()).toString();
	}

	private static final Pattern errorPattern = Pattern.compile("(.+)\\((\\d+)\\)\\s*: error C(\\d+): (.+)");
	private static final Pattern fatalErrorPattern = Pattern.compile("(.+)\\((\\d+)\\)\\s*: fatal error C(\\d+): (.+)");
	private static final Pattern warningPattern = Pattern.compile("(.+)\\((\\d+)\\)\\s*: warning C(\\d+): (.+)");

	@Override
	public void onStdOutLineWrite(String line) {
		if (isFirstOutputLine) {
			isFirstOutputLine = false;
			if (line.equals(filename)) {
				return;
			}
		}
		if (verboseBuild) {
			System.out.println(line);
		}
		Matcher matcher;
		if ((matcher = errorPattern.matcher(line)).matches()) {
			final AbsoluteFilePath filePath = new AbsoluteFilePath(Machine.getLocal(), matcher.group(1));
			final int lineNumber = Integer.parseInt(matcher.group(2));
			final int errorNumber = Integer.parseInt(matcher.group(3));
			final String errorMessage = matcher.group(4);
			buildMessages.add(new BuildMessage(BuildMessage.Type.Error, filePath, lineNumber, errorNumber, errorMessage));
		} else if ((matcher = fatalErrorPattern.matcher(line)).find()) {
			final AbsoluteFilePath filePath = new AbsoluteFilePath(Machine.getLocal(), matcher.group(1));
			final int lineNumber = Integer.parseInt(matcher.group(2));
			final int errorNumber = Integer.parseInt(matcher.group(3));
			final String errorMessage = matcher.group(4);
			buildMessages.add(new BuildMessage(BuildMessage.Type.FatalError, filePath, lineNumber, errorNumber, errorMessage));
		} else if ((matcher = warningPattern.matcher(line)).find()) {
			final AbsoluteFilePath warningPath = new AbsoluteFilePath(Machine.getLocal(), matcher.group(1));
			final int lineNumber = Integer.parseInt(matcher.group(2));
			final int warningNumber = Integer.parseInt(matcher.group(3));
			final String warningMessage = matcher.group(4);
			buildMessages.add(new BuildMessage(BuildMessage.Type.Warning, warningPath, lineNumber, warningNumber, warningMessage));
		} else {
			final BuildMessage lastBuildMessage = buildMessages.getLastBuildMessage();
			if (lastBuildMessage == null) {
				throw new Error(String.format("Unknown compiler message %s", line));
			} else {
				lastBuildMessage.appendMessage("\r\n" + line);
			}
		}
	}

	@Override
	public void onStdErrLineWrite(String line) {
		throw new Error(String.format("Unknown compiler message %s", line));
	}

	@Override
	public void onError(String description) {
		if (verboseBuild) {
			System.err.println(description);
		}
		buildMessages.add(new BuildMessage(BuildMessage.Type.Exception, source, null, null, description));
	}

	@Override
		public void onCompletion(Integer status) {
	}

	private final BuildMessages buildMessages;
	private final boolean verboseBuild;
	private final AbsoluteFilePath source;
	private final String filename;
	private boolean isFirstOutputLine = true;
}
