/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.generic.Versioned;
import info.yeppp.ebuilda.internals.Program;
import info.yeppp.ebuilda.internals.WinRegistry;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class NASM extends AbstractAssembler implements Versioned {

	final static Pattern versionPattern = Pattern.compile("NASM version (\\d(?:\\.\\d+)+) compiled on (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) +(\\d+) +(\\d+)");
	final static String[] months = new String[]{"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

	/** @brief Optimization for branch offsets */
	public static enum Optimization {
		None,
		Minimal,
		Multipass
	}

	private static final ABI[] supportedABI = new ABI[] {
		/* Windows */
			ABI.X86_Windows_Default_i586,
			ABI.X64_Windows_Microsoft_Default,
		/* Mac OS X */
			ABI.X86_MacOSX_Pic_Default,
			ABI.X64_MacOSX_SystemV_Default,
		/* GNU/Linux */
			ABI.X86_Linux_Pic_i586,
			ABI.X64_Linux_SystemV_Default,
		/* Android */
			ABI.X86_Linux_Pic_Android,
		/* Native client */
			ABI.X86_NaCl_NaCl_i686,
			ABI.X64_NaCl_NaCl_Default
	};

	public static ToolInfo verify(AbsoluteFilePath assemblerPath) {
		final List<String> output = Program.getStdoutLines(assemblerPath, new Program.Parameters("-v"), new Program.Environment(), 1);
		if (output.isEmpty()) {
			return new ToolInfo(assemblerPath);
		} else {
			final Matcher versionMatcher = versionPattern.matcher(output.get(0));
			if (versionMatcher.matches()) {
				final String versionString = versionMatcher.group(1);
				final String monthString = versionMatcher.group(2);
				int month = 0;
				for (; month < NASM.months.length; month++) {
					if (NASM.months[month].equals(monthString)) {
						break;
					}
				}
				final String dateString = versionMatcher.group(3);
				final String yearString = versionMatcher.group(4);
				final Version version = new Version(versionString + "." + yearString + "." + Integer.toString(month) + "." + dateString);
				ABI defaultABI = ABI.X86_Linux_Pic_i586;
				final OperatingSystem operatingSystem = Machine.getLocal().getOperatingSystem();
				final Architecture architecture = Machine.getLocal().getArchitecture();
				switch (operatingSystem) {
					case Windows:
						defaultABI = architecture.equals(Architecture.X64) ? ABI.X64_Windows_Microsoft_Default : ABI.X86_Windows_Default_i586; break;
					case Linux:
						defaultABI = architecture.equals(Architecture.X64) ? ABI.X64_Linux_SystemV_Default : ABI.X86_Linux_Pic_i586; break;
					case MacOSX:
						defaultABI = architecture.equals(Architecture.X64) ? ABI.X64_MacOSX_SystemV_Default : ABI.X86_MacOSX_Pic_Default; break;
				}
				return new ToolInfo(assemblerPath, defaultABI, NASM.supportedABI, version);
			} else {
				return new ToolInfo(assemblerPath);
			}
		}
	}

	private static Map<ABI, List<ToolInfo>> initLocalBinaries() {
		final String executableExtension = Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows) ? ".exe" : "";
		final HashSet<AbsoluteDirectoryPath> candidateDirectories = new HashSet<AbsoluteDirectoryPath>(Machine.getLocal().getStandardBinaryDirectories());
		if (Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows)) {
			try {
				final String installPath = WinRegistry.readString(WinRegistry.HKEY_CURRENT_USER, "Software\\nasm", "");
				if (installPath != null) {
					final AbsoluteDirectoryPath installDirectory = new AbsoluteDirectoryPath(Machine.getLocal(), installPath);
					candidateDirectories.add(installDirectory);
				}
			} catch (Exception e) {
			}
		}
		final List<ToolInfo> tools = new ArrayList<ToolInfo>(candidateDirectories.size());
		for (final AbsoluteDirectoryPath candidateDirectory : candidateDirectories) {
			final AbsoluteFilePath assemblerPath = new AbsoluteFilePath(candidateDirectory, new RelativeFilePath("nasm" + executableExtension));
			final ToolInfo assemblerInfo = NASM.verify(assemblerPath);
			if (assemblerInfo.isValid() && (assemblerInfo.getSupportedABI() != null)) {
				tools.add(assemblerInfo);
			}
		}
		final HashMap<ABI, List<ToolInfo>> abiMap = new HashMap<ABI, List<ToolInfo>>(ABI.values().length);
		for (final ABI abi : ABI.values()) {
			abiMap.put(abi, new LinkedList<ToolInfo>());
		}
		for (final ToolInfo tool : tools) {
			for (final ABI abi : tool.getSupportedABI()) {
				abiMap.get(abi).add(tool);
			}
		}
		return abiMap;
	}

	private static Map<ABI, List<ToolInfo>> localBinaries = NASM.initLocalBinaries();

	public static VersionList<NASM> enumerate(Machine machine, ABI abi) {
		final VersionList<NASM> tools = new VersionList<NASM>();
		for (final ToolInfo toolInfo : localBinaries.get(abi)) {
			final NASM toolchain = new NASM(toolInfo.getPath(), toolInfo.getVersion(), abi);
			tools.add(toolchain);
		}
		return tools;
	}

	static {
		Arrays.sort(NASM.supportedABI);
	}

	public static ABI[] getSupportedABI() {
		final ABI[] supportedABI = new ABI[NASM.supportedABI.length];
		System.arraycopy(NASM.supportedABI, 0, supportedABI, 0, NASM.supportedABI.length);
		return supportedABI;
	}

	private NASM(AbsoluteFilePath assemblerPath, Version version, ABI abi) {
		super(abi);
		this.assemblerPath = assemblerPath;
		this.version = version;
		switch (abi.getArchitecture()) {
			case X86:
				switch (abi.getOperatingSystem()) {
					case Windows:
						outputFormat = "win32";
						outputExtension = "obj";
						debugFormat = null;
						break;
					case Linux:
					case NativeClient:
						outputFormat = "elf32";
						outputExtension = "o";
						debugFormat = "dwarf";
						break;
					case MacOSX:
						outputFormat = "macho32";
						outputExtension = "o";
						debugFormat = null;
						break;
					default:
						throw new Error(String.format("Unknown target OS %s for %s architecture", abi.getOperatingSystem().toString(), abi.getArchitecture().toString()));
				}
				break;
			case X64:
				switch (abi.getOperatingSystem()) {
					case Windows:
						outputFormat = "win64";
						outputExtension = "obj";
						debugFormat = null;
						break;
					case Linux:
					case NativeClient:
						outputFormat = "elf64";
						outputExtension = "o";
						debugFormat = "dwarf";
						break;
					case MacOSX:
						outputFormat = "macho64";
						outputExtension = "o";
						debugFormat = null;
						break;
					default:
						throw new Error(String.format("Unknown target OS %s for %s architecture", abi.getOperatingSystem().toString(), abi.getArchitecture().toString()));
				}
				break;
			default:
				throw new Error(String.format("Unsupported target architecture %s", abi.getArchitecture().toString()));
		}
	}

	public NASM(NASM prototype) {
		super(prototype);
		this.assemblerPath = prototype.assemblerPath;
		this.version = prototype.version;
		this.outputFormat = prototype.outputFormat;
		this.outputExtension = prototype.outputExtension;
		this.debugFormat = prototype.debugFormat;
	}

	public BuildMessages assemble(final AbsoluteFilePath source) {
		final BuildMessages buildMessages = new BuildMessages();
		final AbsoluteFilePath object = getObjectPath(source);
		final boolean verboseBuild = isVerboseBuild();

		object.getDirectory().create();

		final Program.Parameters parameters = prepareParameters(source, object);
		final Program.Environment environment = new Program.Environment();
		final Program.EventHandler eventHandler = new Program.EventHandler() {
			@Override
			public void onStdOutLineWrite(String line) {
				if (verboseBuild) {
					System.out.println(line);
				}
			}

			@Override
			public void onStdErrLineWrite(String line) {
				if (line.startsWith(source.getCanonicalForm())) {
					line = line.substring(source.getCanonicalForm().length());
					final Pattern errorPattern = Pattern.compile(":(\\d+): error: (.+)");
					final Matcher errorMatcher = errorPattern.matcher(line);
					if (errorMatcher.find()) {
						int lineNumber = Integer.parseInt(errorMatcher.group(1));
						final String description = errorMatcher.group(2);
						buildMessages.add(new BuildMessage(BuildMessage.Type.Error, source, lineNumber, null, description));
					} else {
						if (verboseBuild) {
							System.err.println(line);
						}
					}
				} else {
					if (verboseBuild) {
						System.err.println(line);
					}
				}
			}

			@Override
			public void onError(String description) {
				if (verboseBuild) {
					System.err.println(description);
				}
				buildMessages.add(new BuildMessage(BuildMessage.Type.Exception, source, null, null, description));
			}

			@Override
			public void onCompletion(Integer status) {
			}
		};
		if (isVerboseBuild()) {
			System.out.println(assemblerPath + " " + parameters.toString());
		}
		final Program assembler = new Program(assemblerPath, parameters, environment, eventHandler);
		assembler.join();
		return buildMessages;
	}

	protected Program.Parameters prepareParameters(AbsoluteFilePath sourcePath, AbsoluteFilePath objectPath) {
		final Program.Parameters parameters = new Program.Parameters();
		parameters.add("-f", this.outputFormat);
		parameters.add("-g");
		if (this.debugFormat != null) {
			parameters.add("-F", this.debugFormat);
		}
		for (final Map.Entry<String, String> macro : this.macros.entrySet()) {
			parameters.add("-D" + macro.getKey() + "=" + macro.getValue());
		}
		if (this.optimization != null) {
			switch (this.optimization) {
				case None:
					parameters.add("-O0"); break;
				case Minimal:
					parameters.add("-O1"); break;
				case Multipass:
					parameters.add("-Ox"); break;
			}
		}
		parameters.add("-o", objectPath.getCanonicalForm());
		parameters.add(sourcePath.getCanonicalForm());
		return parameters;
	}

	public final Optimization getOptimization() {
		return this.optimization;
	}

	public final void setOptimization(Optimization optimization) {
		this.optimization = optimization;
	}

	@Override
	public final Version getVersion() {
		return this.version;
	}

	public final AbsoluteFilePath getExecutablePath() {
		return this.assemblerPath;
	}

	@Override
	public String toString() {
		return "NASM version " + this.version.toString() + " for " + getABI().toString();
	}

	@Override
	public NASM copy() {
		return new NASM(this);
	}

	private final AbsoluteFilePath assemblerPath;
	private final Version version;
	private String outputFormat;
	private String debugFormat;
	private String outputExtension;
	private Optimization optimization = null;
}
