package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.FileName;

import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ToolsetInfo {
	public static List<ToolsetInfo> enumerate(AbsoluteDirectoryPath directory, Pattern prefixPattern, Pattern suffixPattern, String[] filenames) {
		final List<ToolsetInfo> products = new LinkedList<ToolsetInfo>();
		if (filenames.length != 0) {
			final String filename = filenames[0];
			final Pattern testPattern = Pattern.compile(".*" + Pattern.quote(filename) + ".*");
			final List<AbsoluteFilePath> testExecutables = directory.getExecutables(testPattern, false);
			if (!testExecutables.isEmpty()) {
				final List<AbsoluteFilePath> executables = directory.getExecutables(Pattern.compile(".*"), false);
				for (final AbsoluteFilePath executable : executables) {
					final String baseFileName = executable.getFileName().removeExtension(".exe").toString();
					final int filenamePosition = baseFileName.indexOf(filename);
					if (filenamePosition != -1) {
						final String prefixCandidate = baseFileName.substring(0, filenamePosition);
						final Matcher prefixMatcher = prefixPattern.matcher(prefixCandidate);
						if (prefixMatcher.matches()) {
							final String suffixCandidate = baseFileName.substring(filenamePosition + filename.length());
							final Matcher suffixMatcher = suffixPattern.matcher(suffixCandidate);
							if (suffixMatcher.matches()) {
								final String prefix = prefixMatcher.group();
								final String suffix = suffixMatcher.group();
								boolean missFilename = false;
								for (int i = 1; i < filenames.length; i++) {
									final FileName targetFilename = new FileName(prefix + filenames[i] + suffix);
									if (!findFileName(executables, targetFilename)) {
										missFilename = true;
										break;
									}
								}
								if (!missFilename) {
									products.add(new ToolsetInfo(directory, prefix, suffix));
								}
							}
						}
					}
				}
			}
		}
		return products;
	}

	private static boolean findFileName(Collection<AbsoluteFilePath> files, FileName filename) {
		if (files.isEmpty()) {
			return false;
		} else {
			if (files.iterator().next().getMachine().getOperatingSystem().equals(OperatingSystem.Windows)) {
				for (final AbsoluteFilePath file : files) {
					if (file.getFileName().removeExtension(false).equals(filename)) {
						return true;
					}
				}
			} else {
				for (final AbsoluteFilePath file : files) {
					if (file.getFileName().equals(filename)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	public ToolsetInfo(AbsoluteDirectoryPath path, String prefix, String suffix, ABI defaultABI, ABI[] supportedABI, Version version) {
		this.path = path;
		this.prefix = prefix;
		this.suffix = suffix;
		this.defaultABI = defaultABI;
		this.supportedABI = supportedABI == null ? null : new ABI[supportedABI.length];
		if (supportedABI != null) {
			System.arraycopy(supportedABI, 0, this.supportedABI, 0, supportedABI.length);
			if (supportedABI.length > 1) {
				Arrays.sort(supportedABI);
			}
		}
		this.version = version;
	}

	public ToolsetInfo(AbsoluteDirectoryPath path, String prefix, String suffix) {
		this.path = path;
		this.prefix = prefix;
		this.suffix = suffix;
		this.defaultABI = null;
		this.supportedABI = null;
		this.version = null;
	}

	public AbsoluteDirectoryPath getPath() {
		return this.path;
	}

	public String getPrefix() {
		return this.prefix;
	}

	public String getSuffix() {
		return this.suffix;
	}

	public boolean isValid() {
		return this.version != null;
	}

	public boolean isSupported(ABI abi) {
		if (this.supportedABI == null) {
			return false;
		} else if (this.supportedABI.length == 1) {
			return this.supportedABI[0].equals(abi);
		} else {
			return Arrays.binarySearch(this.supportedABI, abi) >= 0;
		}
	}

	public ABI getDefaultABI() {
		return this.defaultABI;
	}

	public ABI[] getSupportedABI() {
		if (this.supportedABI == null) {
			return null;
		} else {
			final ABI[] supportedABI = new ABI[this.supportedABI.length];
			System.arraycopy(this.supportedABI, 0, supportedABI, 0, this.supportedABI.length);
			return supportedABI;
		}
	}

	public Version getVersion() {
		return this.version;
	}

	@Override
	public int hashCode() {
		return path.hashCode() ^ prefix.hashCode() ^ suffix.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof ToolsetInfo) {
			final ToolsetInfo toolsetInfo = (ToolsetInfo)obj;
			return path.equals(toolsetInfo.path) && prefix.equals(toolsetInfo.prefix) && suffix.equals(toolsetInfo.suffix);
		} else {
			return false;
		}
	}

	final AbsoluteDirectoryPath path;
	final String prefix;
	final String suffix;
	final ABI defaultABI;
	final ABI[] supportedABI;
	final Version version;
}
