/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.internals.WinRegistry;
import info.yeppp.ebuilda.WindowsSDK;

import java.util.*;

public class VisualStudio2008 extends VisualStudio {

	private static Map<ABI, List<ToolsetInfo>> initLocalBinaries() {
		final List<ToolsetInfo> toolsets = new ArrayList<ToolsetInfo>();
		if (Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows)) {
			try {
				final String registryPath = Machine.getLocal().getArchitecture().is64bit() ?
						"Software\\WOW6432Node\\Microsoft\\VisualStudio\\9.0":
						"Software\\Microsoft\\VisualStudio\\9.0";
				final String idePath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, registryPath, "InstallDir");

				final AbsoluteDirectoryPath ideDirectory = new AbsoluteDirectoryPath(Machine.getLocal(), idePath);
				final AbsoluteDirectoryPath rootDirectory = new AbsoluteDirectoryPath(ideDirectory, new RelativeDirectoryPath("../.."));
				if (rootDirectory.contains(new RelativeFilePath("VC/include/sal.h"))) {
					final ABI[] supportedABI = enumerateABI(rootDirectory);
					toolsets.add(new ToolsetInfo(rootDirectory, null, null, null, supportedABI, new Version(9, 0)));
				}
			} catch (Exception e) {
			}
		}
		final HashMap<ABI, List<ToolsetInfo>> abiMap = new HashMap<ABI, List<ToolsetInfo>>(ABI.values().length);
		for (final ABI abi : ABI.values()) {
			abiMap.put(abi, new LinkedList<ToolsetInfo>());
		}
		for (final ToolsetInfo toolset : toolsets) {
			for (final ABI abi : toolset.getSupportedABI()) {
				abiMap.get(abi).add(toolset);
			}
		}
		return abiMap;
	}

	private static Map<ABI, List<ToolsetInfo>> localBinaries = VisualStudio2008.initLocalBinaries();

	private static ABI[] enumerateABI(AbsoluteDirectoryPath rootDirectory) {
		final List<ABI> abiList = new ArrayList<ABI>(3);
		if (rootDirectory.contains(new RelativeFilePath("VC/Lib/msvcrt.lib"))) {
			abiList.add(ABI.X86_Windows_Default_i586);
		}
		if (rootDirectory.contains(new RelativeFilePath("VC/Lib/amd64/msvcrt.lib"))) {
			abiList.add(ABI.X64_Windows_Microsoft_Default);
		}
		if (rootDirectory.contains(new RelativeFilePath("VC/Lib/ia64/msvcrt.lib"))) {
			abiList.add(ABI.IA64_Windows_Microsoft_Default);
		}
		final ABI[] abiArray = new ABI[abiList.size()];
		abiList.toArray(abiArray);
		return abiArray;
	}

	public static VersionList<VisualStudio> enumerate(Machine machine, ABI abi) {
		final VersionList<VisualStudio> visualStudioList = new VersionList<VisualStudio>();
		for (final ToolsetInfo toolset : localBinaries.get(abi)) {
			final VisualStudio visualStudio = new VisualStudio2008(toolset.getPath(), findWindowsSDK(machine, abi), abi, toolset.getVersion());
			visualStudioList.add(visualStudio);
		}
		return visualStudioList;
	}

	private static AbsoluteDirectoryPath getCppToolsDirectory(AbsoluteDirectoryPath rootDirectory, ABI abi) {
		final Architecture architecture = rootDirectory.getMachine().getArchitecture();
		switch (abi) {
			case X86_Windows_Default_i586:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("VC/bin"));
			case X64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory,
						new RelativeDirectoryPath(architecture.equals(Architecture.X64) ? "VC/bin/amd64" : "VC/bin/x86_amd64"));
			case IA64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("VC/bin/x86_ia64"));
			default:
				throw new Error(String.format("The ABI %s is not supported by Visual Studio 2008", abi.toString()));
		}
	}

	private static AbsoluteDirectoryPath getCppLibraryDirectory(AbsoluteDirectoryPath rootDirectory, ABI abi) {
		switch (abi) {
			case X86_Windows_Default_i586:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("VC/lib"));
			case X64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("VC/lib/amd64"));
			case IA64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("VC/lib/ia64"));
			default:
				throw new Error(String.format("The ABI %s is not supported by Visual Studio 2008", abi.toString()));
		}
	}

	private VisualStudio2008(AbsoluteDirectoryPath rootDirectory, WindowsSDK windowsSDK, ABI abi, Version version) {
		super(rootDirectory,
				new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("VC/Include")),
				getCppLibraryDirectory(rootDirectory, abi),
				getCppToolsDirectory(rootDirectory, abi),
				windowsSDK, abi, version);
	}

}
