/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.*;
import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.internals.WinRegistry;

import java.util.*;

public class Windows7SDK extends WindowsSDK {

	private static Map<ABI, List<ToolsetInfo>> initLocalBinaries() {
		final List<ToolsetInfo> toolsets = new ArrayList<ToolsetInfo>();
		if (Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows)) {
			try {
				final String registryPath = "Software\\Microsoft\\Microsoft SDKs\\Windows\\v7.0A";
				final String installPath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, registryPath, "InstallationFolder");
				final String versionString = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, registryPath, "ProductVersion");

				final Version version = new Version(versionString);
				final AbsoluteDirectoryPath installDirectory = new AbsoluteDirectoryPath(Machine.getLocal(), installPath);
				if (installDirectory.exists()) {
					if (installDirectory.contains(new RelativeFilePath("Include/Windows.h"))) {
						final ABI[] supportedABI = enumerateABI(installDirectory);
						toolsets.add(new ToolsetInfo(installDirectory, null, null, null, supportedABI, version));
					}
				}
			} catch (Exception e) {
			}
		}
		final HashMap<ABI, List<ToolsetInfo>> abiMap = new HashMap<ABI, List<ToolsetInfo>>(ABI.values().length);
		for (final ABI abi : ABI.values()) {
			abiMap.put(abi, new LinkedList<ToolsetInfo>());
		}
		for (final ToolsetInfo toolset : toolsets) {
			for (final ABI abi : toolset.getSupportedABI()) {
				abiMap.get(abi).add(toolset);
			}
		}
		return abiMap;
	}

	private static Map<ABI, List<ToolsetInfo>> localBinaries = Windows7SDK.initLocalBinaries();

	private static ABI[] enumerateABI(AbsoluteDirectoryPath rootDirectory) {
		final List<ABI> abiList = new ArrayList<ABI>(3);
		if (rootDirectory.contains(new RelativeFilePath("Lib/Kernel32.Lib"))) {
			abiList.add(ABI.X86_Windows_Default_i586);
		}
		if (rootDirectory.contains(new RelativeFilePath("Lib/x64/Kernel32.Lib"))) {
			abiList.add(ABI.X64_Windows_Microsoft_Default);
		}
		if (rootDirectory.contains(new RelativeFilePath("Lib/IA64/Kernel32.Lib"))) {
			abiList.add(ABI.IA64_Windows_Microsoft_Default);
		}
		final ABI[] abiArray = new ABI[abiList.size()];
		abiList.toArray(abiArray);
		return abiArray;
	}

	public static List<WindowsSDK> enumerate(Machine machine, ABI abi) {
		final List<WindowsSDK> sdkList = new ArrayList<WindowsSDK>(1);
		for (final ToolsetInfo toolset : localBinaries.get(abi)) {
			final WindowsSDK sdk = new Windows7SDK(toolset.getPath(), abi, toolset.getVersion());
			sdkList.add(sdk);
		}
		return sdkList;
	}

	private static AbsoluteDirectoryPath getLibraryDirectory(AbsoluteDirectoryPath rootDirectory, ABI abi) {
		switch (abi) {
			case X86_Windows_Default_i586:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Lib"));
			case X64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Lib/x64"));
			case IA64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Lib/IA64"));
			default:
				throw new Error(String.format("The ABI %s is not supported by Windows 7 SDK", abi.toString()));
		}
	}

	protected Windows7SDK(AbsoluteDirectoryPath rootDirectory, ABI abi, Version version) {
		super(rootDirectory,
				new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Include")),
				getLibraryDirectory(rootDirectory, abi),
				abi, version);
	}
}
