/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda;

import info.yeppp.ebuilda.*;
import info.yeppp.ebuilda.filesystem.AbsoluteDirectoryPath;
import info.yeppp.ebuilda.filesystem.AbsoluteFilePath;
import info.yeppp.ebuilda.filesystem.RelativeDirectoryPath;
import info.yeppp.ebuilda.filesystem.RelativeFilePath;
import info.yeppp.ebuilda.internals.WinRegistry;

import java.util.*;

public class Windows8SDK extends WindowsSDK {

	private static Map<ABI, List<ToolsetInfo>> initLocalBinaries() {
		final List<ToolsetInfo> toolsets = new ArrayList<ToolsetInfo>();
		if (Machine.getLocal().getOperatingSystem().equals(OperatingSystem.Windows)) {
			try {
				final String registryPath = Machine.getLocal().getArchitecture().is64bit() ?
						"Software\\Wow6432Node\\Microsoft\\Microsoft SDKs\\Windows\\v8.0":
						"Software\\Microsoft\\Microsoft SDKs\\Windows\\v8.0";
				final String installPath = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, registryPath, "InstallationFolder");
				final String versionString = WinRegistry.readString(WinRegistry.HKEY_LOCAL_MACHINE, registryPath, "ProductVersion");
				/* Some WTF programmer at Microsoft put two dots into version name */
				final String versionStringCorrected = versionString.replace("..", ".");

				final Version version = new Version(versionStringCorrected);
				final AbsoluteDirectoryPath installDirectory = new AbsoluteDirectoryPath(Machine.getLocal(), installPath);
				if (installDirectory.exists()) {
					if (installDirectory.contains(new RelativeFilePath("Include/um/Windows.h"))) {
						if (installDirectory.contains(new RelativeFilePath("Include/shared/sal.h"))) {
							final ABI[] supportedABI = enumerateABI(installDirectory);
							toolsets.add(new ToolsetInfo(installDirectory, null, null, null, supportedABI, version));
						}
					}
				}
			} catch (Exception e) {
			}
		}
		final HashMap<ABI, List<ToolsetInfo>> abiMap = new HashMap<ABI, List<ToolsetInfo>>(ABI.values().length);
		for (final ABI abi : ABI.values()) {
			abiMap.put(abi, new LinkedList<ToolsetInfo>());
		}
		for (final ToolsetInfo toolset : toolsets) {
			for (final ABI abi : toolset.getSupportedABI()) {
				abiMap.get(abi).add(toolset);
			}
		}
		return abiMap;
	}

	private static Map<ABI, List<ToolsetInfo>> localBinaries = Windows8SDK.initLocalBinaries();

	private static ABI[] enumerateABI(AbsoluteDirectoryPath rootDirectory) {
		/* Check that shared include directory exists on disk. Otherwise the Windows 8 SDK is not usable. */
		final List<ABI> abiList = new ArrayList<ABI>(3);
		if (rootDirectory.contains(new RelativeFilePath("Lib/win8/um/x86/kernel32.Lib"))) {
			abiList.add(ABI.X86_Windows_Default_i586);
		}
		if (rootDirectory.contains(new RelativeFilePath("Lib/win8/um/x64/kernel32.Lib"))) {
			abiList.add(ABI.X64_Windows_Microsoft_Default);
		}
		if (rootDirectory.contains(new RelativeFilePath("Lib/win8/um/arm/kernel32.Lib"))) {
			abiList.add(ABI.ARM_Windows_HardEABI_Default);
		}
		final ABI[] abiArray = new ABI[abiList.size()];
		abiList.toArray(abiArray);
		return abiArray;
	}

	public static List<WindowsSDK> enumerate(Machine machine, ABI abi) {
		final List<WindowsSDK> sdkList = new ArrayList<WindowsSDK>(1);
		for (final ToolsetInfo toolset : localBinaries.get(abi)) {
			final WindowsSDK sdk = new Windows8SDK(toolset.getPath(), abi, toolset.getVersion());
			sdkList.add(sdk);
		}
		return sdkList;
	}

	private static AbsoluteDirectoryPath getLibraryDirectory(AbsoluteDirectoryPath rootDirectory, ABI abi) {
		switch (abi) {
			case X86_Windows_Default_i586:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Lib/win8/um/x86"));
			case X64_Windows_Microsoft_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Lib/win8/um/x64"));
			case ARM_Windows_HardEABI_Default:
				return new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Lib/win8/um/arm"));
			default:
				throw new Error(String.format("The ABI %s is not supported by Windows 8 SDK", abi.toString()));
		}
	}

	protected Windows8SDK(AbsoluteDirectoryPath rootDirectory, ABI abi, Version version) {
		super(rootDirectory,
				new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Include/um")),
				getLibraryDirectory(rootDirectory, abi),
				abi, version);
		this.sharedIncludeDirectory = new AbsoluteDirectoryPath(rootDirectory, new RelativeDirectoryPath("Include/shared/"));
	}

	@Override
	protected AbsoluteFilePath getResourceCompilerPath() {
		switch (getABI()) {
			case X86_Windows_Default_i586:
				return new AbsoluteFilePath(getToolsDirectory(), new RelativeFilePath("x86/rc.exe"));
			case X64_Windows_Microsoft_Default:
				return new AbsoluteFilePath(getToolsDirectory(), new RelativeFilePath("x64/rc.exe"));
			case ARM_Windows_HardEABI_Default:
				return new AbsoluteFilePath(getToolsDirectory(), new RelativeFilePath("arm/rc.exe"));
			default:
				throw new Error(String.format("The ABI %s is not supported by Windows 8 SDK", abi.toString()));
		}
	}

	@Override
	public Collection<AbsoluteDirectoryPath> getIncludeDirectories() {
		final List<AbsoluteDirectoryPath> includeDirectories = new LinkedList<AbsoluteDirectoryPath>();
		includeDirectories.add(this.includeDirectory);
		includeDirectories.add(this.sharedIncludeDirectory);
		return includeDirectories;
	}

	private final AbsoluteDirectoryPath sharedIncludeDirectory;
}
