/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda.filesystem;

import info.yeppp.ebuilda.Machine;
import info.yeppp.ebuilda.OperatingSystem;

import java.io.*;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;

public final class AbsoluteFilePath {

	public AbsoluteFilePath(Machine machine, String path) {
		this.machine = machine;
		if (path == null) {
			throw new NullPointerException("The path string is null");
		}
		if (path.isEmpty()) {
			throw new PathFormatException("The path string is empty");
		}
		/* The internal representation always uses POSIX file separators.
		 * Convert Windows path separators '/' into POSIX path separators '\' */
		path = path.replace('\\', '/');
		if (path.endsWith("/")) {
			throw new PathFormatException(String.format("The string %s is not file, but directory path", path));
		}
		if (getMachine().getOperatingSystem().equals(OperatingSystem.Windows)) {
			if (Pattern.matches("[A-Za-z]:/.*", path)) {
				/* Uppercase the drive letter */
				path = path.substring(0, 1).toUpperCase(Locale.US) + path.substring(1);
				final String driveRelativePath = path.substring(3);
				/* Check for forbidden symbols on Windows
				 * Information about forbidden symbols from
				 * http://msdn.microsoft.com/en-us/library/windows/desktop/aa365247.aspx
				 */
				checkForForbiddenWindowsPathSymbol(driveRelativePath, '<');
				checkForForbiddenWindowsPathSymbol(driveRelativePath, '>');
				checkForForbiddenWindowsPathSymbol(driveRelativePath, ':');
				checkForForbiddenWindowsPathSymbol(driveRelativePath, '"');
				checkForForbiddenWindowsPathSymbol(driveRelativePath, '|');
				checkForForbiddenWindowsPathSymbol(driveRelativePath, '?');
				checkForForbiddenWindowsPathSymbol(driveRelativePath, '*');
				final String[] components = path.split("/");

				/* Verify the filename */
				checkForForbiddenWindowsFileName(components[components.length - 1]);

				/* Remove empty directories (C:\\\filename.txt),
				 * references to current directory (C:\.\filename.txt),
				 * and references to the parent directory (C:\parent\..\filename.txt)
				 */
				final List<String> directories = new ArrayList<String>(components.length);
				for (int i = 1; i < components.length - 1; i++) {
					final String component = components[i];
					if (component.equals(".")) {
						continue;
					} else if (component.equals("..")) {
						if (directories.isEmpty()) {
							/* Reference to parent directory, but no actual parent directory in the path (C:\parent\..\..\filename.txt) */
							throw new PathFormatException(String.format("The path %s has imbalanced references to parent directories (..)", path));
						} else {
							directories.remove(directories.size() - 1);
						}
					} else {
						directories.add(component);
					}
				}
				/* Make the final path */
				final StringBuilder stringBuilder = new StringBuilder(directories.size() * 2 + 2);
				stringBuilder.append(path.substring(0, 3));
				for (final String directory : directories) {
					stringBuilder.append(directory);
					stringBuilder.append('/');
				}
				stringBuilder.append(components[components.length - 1]);
				this.path = stringBuilder.toString();
			} else {
				throw new PathFormatException(String.format("The string %s is not an absolute file path on Windows: disk drive is not specified", path));
			}
		} else {
			if (path.startsWith("//")) {
				throw new PathFormatException(String.format("The UNC path %s is not supported", path));
			}

			if (path.startsWith("/")) {
				checkForForbiddenUnixPathSymbol(path, '<');
				checkForForbiddenUnixPathSymbol(path, '>');
				checkForForbiddenUnixPathSymbol(path, '"');
				checkForForbiddenUnixPathSymbol(path, '|');
				checkForForbiddenUnixPathSymbol(path, '?');
				checkForForbiddenUnixPathSymbol(path, '*');
				final String[] components = path.split("/");

				/* Remove empty directories (/home/user//filename.txt),
				 * references to current directory (/home/user/./filename.txt),
				 * and references to the parent directory (/home/user/parent/../filename.txt)
				 */
				final List<String> directories = new ArrayList<String>(components.length);
				for (int i = 1; i < components.length - 1; i++) {
					final String component = components[i];
					if (component.equals(".") || component.isEmpty()) {
						continue;
					} else if (component.equals("..")) {
						if (directories.isEmpty()) {
							/* Reference to parent directory, but no actual parent directory in the path (/home/user/parent/../../../../filename.txt) */
							throw new PathFormatException(String.format("The path %s has imbalanced references to parent directories (..)", path));
						} else {
							directories.remove(directories.size() - 1);
						}
					} else {
						directories.add(component);
					}
				}
				/* Make the final path */
				final StringBuilder stringBuilder = new StringBuilder(directories.size() * 2 + 2);
				stringBuilder.append('/');
				for (final String directory : directories) {
					stringBuilder.append(directory);
					stringBuilder.append('/');
				}
				stringBuilder.append(components[components.length - 1]);
				this.path = stringBuilder.toString();
			} else {
				throw new PathFormatException(String.format("The string %s is not an absolute file path on Unix", path));
			}
		}
	}

	public AbsoluteFilePath(AbsoluteDirectoryPath directory, RelativeFilePath file) {
		String directoryPath = directory.toString();
		String filePath = file.toString();
		while (filePath.startsWith("../")) {
			final int separatorIndex = directoryPath.lastIndexOf('/', directoryPath.length() - 2); /* Skip the final separator */
			if (separatorIndex == -1) {
				throw new PathFormatException(String.format("The file path %s has too many references to parent directories to combine with directory path %s", file.toString(), directory.toString()));
			} else {
				directoryPath = directoryPath.substring(0, separatorIndex + 1);
				filePath = filePath.substring(3);
			}
		}
		this.machine = directory.machine;
		this.path = directoryPath + filePath;
	}

	public AbsoluteFilePath resolve() {
		try {
			return new AbsoluteFilePath(this.getMachine(), new File(this.getCanonicalForm()).getCanonicalPath());
		} catch (IOException e) {
			return this;
		}
	}

	public RelativeFilePath getRelativePath(AbsoluteDirectoryPath directory) {
		final String[] directoryComponents = directory.toString().split("/");
		final String[] filepathComponents = this.toString().split("/");
		int commonComponentCount = 0;
		for (; commonComponentCount < Math.min(directoryComponents.length, filepathComponents.length); commonComponentCount++) {
			final String directoryComponent = directoryComponents[commonComponentCount];
			final String filepathComponent = filepathComponents[commonComponentCount];
			if (!directoryComponent.equals(filepathComponent)) {
				 break;
			}
		}
		if (commonComponentCount == 0) {
			/* On Windows at least the drive letter must match */
			if (machine.getOperatingSystem().equals(OperatingSystem.Windows)) {
				throw new PathFormatException("The path %s can not be expressed relative to path %s: the paths use different drive letter");
			}
		}
		final StringBuilder components = new StringBuilder();
		for (int i = commonComponentCount; i < directoryComponents.length; i++) {
			components.append("../");
		}
		for (int i = commonComponentCount; i < filepathComponents.length; i++) {
			components.append(filepathComponents[i]);
			if (i != filepathComponents.length - 1) {
				components.append('/');
			}
		}
		return new RelativeFilePath(components.toString());
	}

	public AbsoluteDirectoryPath getDirectory() {
		final int lastSeparator = this.path.lastIndexOf('/');
		if (lastSeparator == -1) {
			throw new Error(String.format("No separator found in the path %s", toString()));
		} else {
			return new AbsoluteDirectoryPath(getMachine(), this.path.substring(0, lastSeparator + 1));
		}
	}

	public FileName getFileName() {
		final int lastSeparator = this.path.lastIndexOf('/');
		if (lastSeparator == -1) {
			throw new Error(String.format("No separator found in the path %s", toString()));
		} else {
			return new FileName(this.path.substring(lastSeparator + 1));
		}
	}

	public static boolean isValid(final Machine machine, final String path) {
		if (path == null) {
			return false;
		} else {
			try {
				new AbsoluteFilePath(machine, path);
				return true;
			} catch (PathFormatException e) {
				return false;
			}
		}
	}

	public boolean exists() {
		if (getMachine() == Machine.getLocal()) {
			final File file = new File(getCanonicalForm());
			return file.exists() && file.isFile();
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	public boolean tryDelete() {
		if (getMachine() == Machine.getLocal()) {
			final File file = new File(getCanonicalForm());
			if (file.isFile()) {
				return file.delete();
			} else {
				return false;
			}
		} else {
			throw new Error("Remote connection is not supported yet");
		}
	}

	public AbsoluteFilePath removeExtension() {
		final int lastPathSeparator = this.path.lastIndexOf('/');
		final int lastExtensionSeparator = this.path.lastIndexOf('.');
		if (lastExtensionSeparator <= lastPathSeparator + 1) {
			throw new PathFormatException(String.format("The path %s does not have an extension to replace", toString()));
		} else {
			return new AbsoluteFilePath(this.machine, this.path.substring(0, lastExtensionSeparator));
		}
	}

	public AbsoluteFilePath tryRemoveExtension() {
		final int lastPathSeparator = this.path.lastIndexOf('/');
		final int lastExtensionSeparator = this.path.lastIndexOf('.');
		if (lastExtensionSeparator <= lastPathSeparator + 1) {
			return this;
		} else {
			return new AbsoluteFilePath(this.machine, this.path.substring(0, lastExtensionSeparator));
		}
	}

	public AbsoluteFilePath replaceExtension(String extension) {
		final int lastPathSeparator = this.path.lastIndexOf('/');
		final int lastExtensionSeparator = this.path.lastIndexOf('.');
		if (lastExtensionSeparator <= lastPathSeparator + 1) {
			throw new PathFormatException(String.format("The path %s does not have an extension to replace", toString()));
		} else {
			return new AbsoluteFilePath(this.machine, this.path.substring(0, lastExtensionSeparator + 1) + extension);
		}
	}

	public AbsoluteFilePath setExtension(String extension) {
		final int lastPathSeparator = this.path.lastIndexOf('/');
		final int lastExtensionSeparator = this.path.lastIndexOf('.');
		if (lastExtensionSeparator <= lastPathSeparator) {
			return new AbsoluteFilePath(this.machine, this.path + "." + extension);
		} else {
			return new AbsoluteFilePath(this.machine, this.path.substring(0, lastExtensionSeparator + 1) + extension);
		}
	}

	public String readFirstLine() {
		try {
			final FileInputStream fis = new FileInputStream(getCanonicalForm());
			final InputStreamReader isr = new InputStreamReader(fis, Charset.defaultCharset());
			final BufferedReader br = new BufferedReader(isr);
			return br.readLine();
		} catch (Exception e) {
			return null;
		}
	}

	static void checkForForbiddenWindowsFileName(final String name) {
		final String[] forbiddenFileNames = new String[]{
				"CON" , "PRN", "AUX", "NUL", "..", ".",
				"COM1", "COM2", "COM3", "COM4", "COM5", "COM6", "COM7", "COM8", "COM9",
				"LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", "LPT9"};
		final String upperCaseName = name.toUpperCase(Locale.US);
		for (int i = 0; i < forbiddenFileNames.length; i++) {
			if (upperCaseName.equals(forbiddenFileNames[i])) {
				throw new PathFormatException(String.format("The file name %s is not allowed on Windows", name));
			}
		}
	}

	static void checkForForbiddenWindowsPathSymbol(final String path, final char symbol) {
		final String symbolString = new String(new char[]{symbol});
		if (path.contains(symbolString)) {
			throw new PathFormatException(String.format("The file path %s contains symbol '%c' which is not allowed on Windows", path, symbol));
		}

	}

	static void checkForForbiddenUnixPathSymbol(final String path, final char symbol) {
		final String symbolString = new String(new char[]{symbol});
		if (path.contains(symbolString)) {
			throw new PathFormatException(String.format("The file path %s contains symbol '%c' which is not allowed on Unix", path, symbol));
		}

	}

	protected final Machine machine;
	private final String path;

	public String getCanonicalForm() {
		if (getMachine().getOperatingSystem().equals(OperatingSystem.Windows)) {
			return this.path.replace('/', '\\');
		} else {
			return this.path;
		}
	}

	public final Machine getMachine() {
		return this.machine;
	}

	@Override
	public String toString() {
		return this.path;
	}

	@Override
	public boolean equals(Object other) {
		if (other == null) {
			return false;
		} else if (other instanceof AbsoluteFilePath) {
			final AbsoluteFilePath otherPath = (AbsoluteFilePath)other;
			return this.path.equals(otherPath.path) && this.machine.equals(otherPath.machine);
		} else if (other instanceof String) {
			return this.path.equals(other) && this.machine.isLocal();
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return this.path.hashCode();
	}
}
