/*
 *                      EBUILDA: cross-platform build system
 *
 * This file is part of Yeppp! library infrastructure and licensed under Simplified BSD License.
 * See LICENSE.txt for the full text of the license.
 *
 */

package info.yeppp.ebuilda.filesystem;

import info.yeppp.ebuilda.Machine;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;

public final class RelativeFilePath {

	public RelativeFilePath(String path) {
		if (path == null) {
			throw new NullPointerException("The path string is null");
		}
		if (path.isEmpty()) {
			throw new PathFormatException("The path string is empty");
		}
		/* Convert Windows path separators '\' into Unix path separators '/' */
		path = path.replace('\\', '/');
		if (path.endsWith("/")) {
			throw new PathFormatException(String.format("The string %s is not file, but directory path", path));
		}
		if (path.startsWith("/")) {
			throw new PathFormatException(String.format("The string %s is not relative, but an absolute path", path));
		}
		if (Pattern.matches("[A-Za-z]:/.*", path)) {
			throw new PathFormatException(String.format("The string %s is not a relative file path on Windows: disk drive is specified", path));
		}

		/* Check for forbidden symbols on Windows or Linux
		 * Information about forbidden symbols from
		 * http://msdn.microsoft.com/en-us/library/windows/desktop/aa365247.aspx
		 */
		checkForForbiddenPathSymbol(path, '<');
		checkForForbiddenPathSymbol(path, '>');
		checkForForbiddenPathSymbol(path, ':');
		checkForForbiddenPathSymbol(path, '"');
		checkForForbiddenPathSymbol(path, '|');
		checkForForbiddenPathSymbol(path, '?');
		checkForForbiddenPathSymbol(path, '*');
		final String[] components = path.split("/");

		/* Check for forbidden file names on Linux */
		checkForForbiddenWindowsFileName(components[components.length - 1]);

		/* Remove empty directories (directory///filename.txt),
		 * references to current directory (directory/./filename.txt),
		 * and references to the parent directory (directory/parent/../filename.txt)
		 */
		final List<String> directories = new ArrayList<String>(components.length);
		for (int i = 0; i < components.length - 1; i++) {
			final String component = components[i];
			if (component.equals(".") || component.isEmpty()) {
				continue;
			} else if (component.equals("..")) {
				if (directories.isEmpty()) {
					directories.add("..");
				} else {
					if (directories.get(directories.size() - 1).equals("..")) {
						directories.add("..");
					} else {
						directories.remove(directories.size() - 1);
					}
				}
			} else {
				directories.add(component);
			}
		}

		/* Make the final path */
		final StringBuilder stringBuilder = new StringBuilder(directories.size() * 2 + 2);
		for (final String directory : directories) {
			stringBuilder.append(directory);
			stringBuilder.append('/');
		}
		stringBuilder.append(components[components.length - 1]);
		this.path = stringBuilder.toString();
	}

	public RelativeFilePath(RelativeDirectoryPath directory, FileName filename) {
		if (directory == null) {
			throw new NullPointerException("The directory object is null");
		}
		if (filename == null) {
			throw new NullPointerException("The filename object is null");
		}
		this.path = directory.toString() + filename.toString();
	}

	public RelativeDirectoryPath getDirectory() {
		final int lastSeparator = this.path.lastIndexOf('/');
		if (lastSeparator == -1) {
			return new RelativeDirectoryPath(".");
		} else {
			return new RelativeDirectoryPath(this.path.substring(0, lastSeparator + 1));
		}
	}

	public FileName getFileName() {
		final int lastSeparator = this.path.lastIndexOf('/');
		if (lastSeparator == -1) {
			return new FileName(this.path);
		} else {
			return new FileName(this.path.substring(lastSeparator + 1));
		}
	}

	public RelativeFilePath replaceExtension(String extension) {
		int lastDotPosition = this.path.lastIndexOf('.');
		/* lastDotPosition is -1 if path contains no dot, and 0 if the file just starts with a dot (e.g. .htaccess) */
		if (lastDotPosition <= 0) {
			throw new PathFormatException(String.format("The path %s does not have an extension to replace", toString()));
		}
		return new RelativeFilePath(this.path.substring(0, lastDotPosition + 1) + extension);
	}

	public boolean isValid(final Machine machine, final String path) {
		if (path == null) {
			return false;
		} else {
			try {
				new RelativeFilePath(path);
				return true;
			} catch (PathFormatException e) {
				return false;
			}
		}
	}

	private void checkForForbiddenWindowsFileName(final String name) {
		final String[] forbiddenFileNames = new String[]{
				"CON" , "PRN", "AUX", "NUL", "..", ".",
				"COM1", "COM2", "COM3", "COM4", "COM5", "COM6", "COM7", "COM8", "COM9",
				"LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", "LPT9"};
		final String upperCaseName = name.toUpperCase(Locale.US);
		for (int i = 0; i < forbiddenFileNames.length; i++) {
			if (upperCaseName.equals(forbiddenFileNames[i])) {
				throw new PathFormatException(String.format("The file name %s is not allowed on Windows", name));
			}
		}
	}

	private void checkForForbiddenPathSymbol(final String path, final char symbol) {
		final String symbolString = new String(new char[]{symbol});
		if (path.contains(symbolString)) {
			throw new PathFormatException(String.format("The file path %s contains symbol '%c' which is not allowed on Windows", path, symbol));
		}

	}

	private void checkForForbiddenUnixPathSymbol(final String path, final char symbol) {
		final String symbolString = new String(new char[]{symbol});
		if (path.contains(symbolString)) {
			throw new PathFormatException(String.format("The file path %s contains symbol '%c' which is not allowed on Unix", path, symbol));
		}

	}

	private final String path;

	@Override
	public String toString() {
		return this.path;
	}

	@Override
	public boolean equals(Object other) {
		if (other == null) {
			return false;
		} else if (other instanceof RelativeFilePath) {
			final RelativeFilePath otherPath = (RelativeFilePath)other;
			return this.path.equals(otherPath.path);
		} else if (other instanceof String) {
			return this.path.equals(other);
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return this.path.hashCode();
	}
}
